#!/usr/bin/env python3
from __future__ import annotations

import requests

from _common import API_ROOT, api_request, build_auth_parser, dump, tenant_access_token


def main() -> int:
    parser = build_auth_parser("Delete Feishu calendar event")
    parser.add_argument("--calendar-id", required=True)
    parser.add_argument("--event-id", required=True)
    args = parser.parse_args()
    token = tenant_access_token(args.app_id, args.app_secret)
    result = api_request(
        "DELETE",
        f"{API_ROOT}/calendar/v4/calendars/{args.calendar_id}/events/{args.event_id}",
        token=token,
    )
    dump(result if result else {"status": "success"})
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
from __future__ import annotations

import argparse
import json
import os
import re
import shutil
import subprocess
import sys
import time
import tempfile
import zipfile
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple

try:
    from .fleet_interactive import (
        interactive_add_bot as _interactive_add_bot,
        interactive_manifest_setup as _interactive_manifest_setup,
    )
    from .fleet_manifest import (
        DEFAULT_MANIFEST_PATH,
        _bot_seed_is_placeholder,
        _child_process_env,
        _dump_json,
        _is_placeholder_text,
        _load_json,
        _normalize_context_settings,
        _required_text,
        _scaffold_manifest,
        _scan_placeholder_fields,
        _validate_webhook_ports,
        prepare_from_manifest,
    )
    from .fleet_process import (
        fleet_web_status_payload as _proc_fleet_web_status_payload,
        is_pid_alive as _proc_is_pid_alive,
        resolve_bot_from_manifest as _proc_resolve_bot_from_manifest,
        _safe_load_json as _proc_safe_load_json,
        start_all_bots_from_manifest as _proc_start_all_bots_from_manifest,
        start_fleet_web as _proc_start_fleet_web,
        start_single_bot_from_manifest as _proc_start_single_bot_from_manifest,
        status_all_bots_from_manifest as _proc_status_all_bots_from_manifest,
        status_single_bot_from_manifest as _proc_status_single_bot_from_manifest,
        stop_all_bots_from_manifest as _proc_stop_all_bots_from_manifest,
        stop_all_fallback_without_manifest as _proc_stop_all_fallback_without_manifest,
        stop_fleet_web as _proc_stop_fleet_web,
        stop_fleet_web_fallback as _proc_stop_fleet_web_fallback,
        stop_single_bot_from_manifest as _proc_stop_single_bot_from_manifest,
    )
    from .fleet_output import (
        format_agent_system_payload,
        format_emergency_stop_payload,
        format_list_bots_payload,
        format_prepare_payload,
        format_restart_payload,
        format_start_stop_payload,
        format_status_payload,
        format_web_payload,
        print_json,
    )
except ImportError:
    from fleet_interactive import (
        interactive_add_bot as _interactive_add_bot,
        interactive_manifest_setup as _interactive_manifest_setup,
    )
    from fleet_manifest import (
        DEFAULT_MANIFEST_PATH,
        _bot_seed_is_placeholder,
        _child_process_env,
        _dump_json,
        _is_placeholder_text,
        _load_json,
        _normalize_context_settings,
        _required_text,
        _scaffold_manifest,
        _scan_placeholder_fields,
        _validate_webhook_ports,
        prepare_from_manifest,
    )
    from fleet_process import (
        fleet_web_status_payload as _proc_fleet_web_status_payload,
        is_pid_alive as _proc_is_pid_alive,
        resolve_bot_from_manifest as _proc_resolve_bot_from_manifest,
        _safe_load_json as _proc_safe_load_json,
        start_all_bots_from_manifest as _proc_start_all_bots_from_manifest,
        start_fleet_web as _proc_start_fleet_web,
        start_single_bot_from_manifest as _proc_start_single_bot_from_manifest,
        status_all_bots_from_manifest as _proc_status_all_bots_from_manifest,
        status_single_bot_from_manifest as _proc_status_single_bot_from_manifest,
        stop_all_bots_from_manifest as _proc_stop_all_bots_from_manifest,
        stop_all_fallback_without_manifest as _proc_stop_all_fallback_without_manifest,
        stop_fleet_web as _proc_stop_fleet_web,
        stop_fleet_web_fallback as _proc_stop_fleet_web_fallback,
        stop_single_bot_from_manifest as _proc_stop_single_bot_from_manifest,
    )
    from fleet_output import (
        format_agent_system_payload,
        format_emergency_stop_payload,
        format_list_bots_payload,
        format_prepare_payload,
        format_restart_payload,
        format_start_stop_payload,
        format_status_payload,
        format_web_payload,
        print_json,
    )

REPO_ROOT = Path(__file__).resolve().parents[1]
SERVICE_PATH = REPO_ROOT / "cheapclaw_service.py"
FLEET_WEB_CONSOLE_PATH = REPO_ROOT / "scripts" / "fleet_web_console.py"
AGENT_LIBRARY_ROOT = REPO_ROOT / "assets" / "agent_library"

# Keep these legacy private names stable because the web console and older
# internal callers still import them from fleet_one_click during upgrades.
_safe_load_json = _proc_safe_load_json
_is_pid_alive = _proc_is_pid_alive
_status_single_bot_from_manifest = _proc_status_single_bot_from_manifest
_status_all_bots_from_manifest = _proc_status_all_bots_from_manifest
_start_single_bot_from_manifest = _proc_start_single_bot_from_manifest
_start_all_bots_from_manifest = _proc_start_all_bots_from_manifest
_stop_single_bot_from_manifest = _proc_stop_single_bot_from_manifest
_stop_all_bots_from_manifest = _proc_stop_all_bots_from_manifest

def _run(cmd: List[str], env: Dict[str, str], check: bool = True) -> int:
    print("[RUN]", " ".join(cmd))
    proc = subprocess.run(cmd, env=env)
    if check and proc.returncode != 0:
        raise SystemExit(proc.returncode)
    return int(proc.returncode)


def _run_capture(cmd: List[str], env: Dict[str, str], check: bool = True) -> subprocess.CompletedProcess:
    proc = subprocess.run(cmd, env=env, capture_output=True, text=True)
    if check and proc.returncode != 0:
        stderr = (proc.stderr or "").strip()
        stdout = (proc.stdout or "").strip()
        detail = stderr or stdout or f"exit code={proc.returncode}"
        raise SystemExit(f"[ERROR] command failed: {' '.join(cmd)}\n{detail}")
    return proc


def _print_log_tail(path: Path, lines: int = 200) -> None:
    if not path.exists():
        print(f"[WARN] log file not found: {path}")
        return
    total = max(1, int(lines))
    try:
        content = path.read_text(encoding="utf-8", errors="ignore").splitlines()
    except Exception as exc:
        raise SystemExit(f"[ERROR] failed to read log: {path} ({exc})")
    print(f"[LOG] {path} (last {total} lines)")
    for line in content[-total:]:
        print(line)


def _install_agent_system_archive(archive_path: Path, destination_root: Path) -> Dict[str, Any]:
    archive = archive_path.expanduser().resolve()
    destination = destination_root.expanduser().resolve()
    if not archive.exists():
        raise SystemExit(f"[ERROR] archive not found: {archive}")
    if archive.suffix.lower() != ".zip":
        raise SystemExit(f"[ERROR] archive must be a .zip file: {archive}")

    with tempfile.TemporaryDirectory(prefix="cheapclaw-agent-system-") as tmpdir:
        tmp_root = Path(tmpdir)
        try:
            with zipfile.ZipFile(archive, "r") as zf:
                zf.extractall(tmp_root)
        except zipfile.BadZipFile as exc:
            raise SystemExit(f"[ERROR] invalid zip archive: {archive} ({exc})")

        entries = [item for item in tmp_root.iterdir() if item.name != "__MACOSX"]
        if len(entries) == 1 and entries[0].is_dir():
            source_root = entries[0]
        else:
            source_root = tmp_root

        level_3 = source_root / "level_3_agents.yaml"
        if not level_3.exists():
            children = [item for item in source_root.iterdir() if item.is_dir()]
            if len(children) == 1 and (children[0] / "level_3_agents.yaml").exists():
                source_root = children[0]
                level_3 = source_root / "level_3_agents.yaml"
        if not level_3.exists():
            raise SystemExit(f"[ERROR] agent system archive missing level_3_agents.yaml: {archive}")

        system_name = source_root.name
        target_dir = destination / system_name
        destination.mkdir(parents=True, exist_ok=True)
        if target_dir.exists():
            shutil.rmtree(target_dir)
        shutil.copytree(source_root, target_dir)

        periodic_variant = ""
        if destination.name == "agent_library" and system_name != "CheapClawSupervisor" and not system_name.endswith("_tmp"):
            previous_root = os.environ.get("MLA_USER_DATA_ROOT", "")
            try:
                os.environ["MLA_USER_DATA_ROOT"] = str(destination.parent)
                try:
                    from .tool_runtime_helpers import ensure_periodic_agent_overlay
                except ImportError:
                    from tool_runtime_helpers import ensure_periodic_agent_overlay
                overlay = ensure_periodic_agent_overlay(str(destination.parent / "cheapclaw" / "supervisor_task"), agent_system=system_name, agent_name="")
                if overlay.get("status") == "success":
                    periodic_variant = str(overlay.get("periodic_agent_system") or "")
            finally:
                if previous_root:
                    os.environ["MLA_USER_DATA_ROOT"] = previous_root
                else:
                    os.environ.pop("MLA_USER_DATA_ROOT", None)

        file_count = sum(1 for item in target_dir.rglob("*") if item.is_file())
        payload = {
            "status": "success",
            "name": system_name,
            "archive": str(archive),
            "target": str(target_dir),
            "files": file_count,
        }
        if periodic_variant:
            payload["periodic_variant"] = periodic_variant
        return payload


def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(description="CheapClaw one-click fleet launcher")
    sub = parser.add_subparsers(dest="command", required=True)

    def _add_json_flag(p: argparse.ArgumentParser) -> None:
        p.add_argument("--json", action="store_true", help="Print machine-readable JSON output")

    def _add_restore_defaults_flag(p: argparse.ArgumentParser) -> None:
        p.add_argument(
            "--restore-package-defaults",
            action="store_true",
            help="Overwrite built-in runtime agent systems from current package assets during prepare/bootstrap",
        )

    p_init = sub.add_parser("init")
    p_init.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_init.add_argument("--force", action="store_true", help="Overwrite manifest if it already exists")
    p_init.add_argument("--open-config", action="store_true", help="Open manifest in editor after init")
    p_init.add_argument("--interactive", action="store_true", help="Configure manifest interactively")

    p_config = sub.add_parser("config")
    p_config.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_config.add_argument("--force", action="store_true", help="Overwrite manifest if it already exists")
    p_config.add_argument("--open-config", action="store_true", help="Open manifest in editor after config")
    p_config.add_argument("--interactive", dest="interactive", action="store_true", default=True, help="Configure manifest interactively (default: true)")
    p_config.add_argument("--no-interactive", dest="interactive", action="store_false", help="Disable interactive setup")

    for name in ("prepare", "up", "run"):
        p = sub.add_parser(name)
        p.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
        p.add_argument("--fleet-config-out", default="", help="Optional output path for generated fleet config")
        p.add_argument("--poll-interval", type=int, default=5, help="Polling interval for fleet run")
        p.add_argument(
            "--init-if-missing",
            dest="init_if_missing",
            action="store_true",
            default=True,
            help="Auto-copy template manifest when manifest is missing (default: true)",
        )
        p.add_argument(
            "--no-init-if-missing",
            dest="init_if_missing",
            action="store_false",
            help="Disable auto init when manifest is missing",
        )
        p.add_argument("--open-config-on-init", action="store_true", help="Open manifest when auto-initialized")
        _add_restore_defaults_flag(p)
        p.add_argument(
            "--interactive",
            action="store_true",
            help="Run interactive setup when manifest is missing or still has placeholders",
        )
        p.add_argument(
            "--foreground",
            action="store_true",
            help="Run in foreground (debug mode). Default starts services in background.",
        )
        p.add_argument(
            "--with-web-console",
            dest="with_web_console",
            action="store_true",
            default=True,
            help="Ensure fleet web console is running (default: true)",
        )
        p.add_argument(
            "--no-web-console",
            dest="with_web_console",
            action="store_false",
            help="Do not start fleet web console automatically",
        )
        p.add_argument("--web-host", default="127.0.0.1", help="Fleet web console host")
        p.add_argument("--web-port", type=int, default=8787, help="Fleet web console port")
        _add_json_flag(p)

    p_list = sub.add_parser("list-bots")
    p_list.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    _add_json_flag(p_list)

    p_status = sub.add_parser("status")
    p_status.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_status.add_argument("--bot-id", default="", help="Optional bot id filter")
    _add_json_flag(p_status)

    p_start_all = sub.add_parser("start")
    p_start_all.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_start_all.add_argument("--bot-id", default="", help="Optional bot id; when omitted starts all enabled bots")
    p_start_all.add_argument("--poll-interval", type=int, default=5, help="Polling interval for started bot process")
    p_start_all.add_argument("--prepare-first", action="store_true", help="Run prepare before starting")
    p_start_all.add_argument(
        "--with-web-console",
        dest="with_web_console",
        action="store_true",
        default=True,
        help="Ensure fleet web console is running (default: true)",
    )
    p_start_all.add_argument(
        "--no-web-console",
        dest="with_web_console",
        action="store_false",
        help="Do not start fleet web console automatically",
    )
    p_start_all.add_argument("--web-host", default="127.0.0.1", help="Fleet web console host")
    p_start_all.add_argument("--web-port", type=int, default=8787, help="Fleet web console port")
    _add_restore_defaults_flag(p_start_all)
    _add_json_flag(p_start_all)

    p_stop_all = sub.add_parser("stop")
    p_stop_all.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_stop_all.add_argument("--bot-id", default="", help="Optional bot id; when omitted stops all bots in manifest")
    p_stop_all.add_argument("--timeout-sec", type=int, default=8, help="Graceful stop timeout before SIGKILL")
    p_stop_all.add_argument(
        "--with-web-console",
        dest="with_web_console",
        action="store_true",
        default=True,
        help="Stop fleet web console together (default: true)",
    )
    p_stop_all.add_argument(
        "--no-web-console",
        dest="with_web_console",
        action="store_false",
        help="Stop bots only and keep fleet web console running",
    )
    _add_json_flag(p_stop_all)

    p_restart = sub.add_parser("restart")
    p_restart.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_restart.add_argument("--bot-id", default="", help="Optional bot id; when omitted restarts all bots")
    p_restart.add_argument("--poll-interval", type=int, default=5, help="Polling interval for started bot process")
    p_restart.add_argument("--timeout-sec", type=int, default=8, help="Graceful stop timeout before SIGKILL")
    p_restart.add_argument("--prepare-first", action="store_true", help="Run prepare before restart")
    p_restart.add_argument(
        "--with-web-console",
        dest="with_web_console",
        action="store_true",
        default=True,
        help="Ensure fleet web console is running (default: true)",
    )
    p_restart.add_argument(
        "--no-web-console",
        dest="with_web_console",
        action="store_false",
        help="Do not start fleet web console automatically",
    )
    p_restart.add_argument("--web-host", default="127.0.0.1", help="Fleet web console host")
    p_restart.add_argument("--web-port", type=int, default=8787, help="Fleet web console port")
    _add_restore_defaults_flag(p_restart)
    _add_json_flag(p_restart)

    p_logs = sub.add_parser("logs")
    p_logs.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_logs.add_argument("--bot-id", required=True, help="Bot id to inspect logs")
    p_logs.add_argument("--lines", type=int, default=200, help="How many lines to print")
    p_logs.add_argument(
        "--kind",
        choices=["loop", "service"],
        default="loop",
        help="loop=service.loop.log, service=cheapclaw_service.log",
    )

    p_add = sub.add_parser("add-bot")
    p_add.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_add.add_argument("--poll-interval", type=int, default=5, help="Polling interval for started bot process")
    p_add.add_argument("--prepare", dest="prepare_after", action="store_true", default=True, help="Run prepare after adding bot (default: true)")
    p_add.add_argument("--no-prepare", dest="prepare_after", action="store_false", help="Do not run prepare after adding bot")
    p_add.add_argument("--start", dest="start_after", action="store_true", default=True, help="Start new bot process after prepare (default: true)")
    p_add.add_argument("--no-start", dest="start_after", action="store_false", help="Do not start new bot process")
    _add_restore_defaults_flag(p_add)
    _add_json_flag(p_add)

    p_start = sub.add_parser("start-bot")
    p_start.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_start.add_argument("--bot-id", required=True, help="Bot id to start")
    p_start.add_argument("--poll-interval", type=int, default=5, help="Polling interval for started bot process")
    p_start.add_argument("--prepare-first", action="store_true", help="Run prepare before starting bot")
    _add_restore_defaults_flag(p_start)
    _add_json_flag(p_start)

    p_stop = sub.add_parser("stop-bot")
    p_stop.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_stop.add_argument("--bot-id", required=True, help="Bot id to stop")
    p_stop.add_argument("--timeout-sec", type=int, default=8, help="Graceful stop timeout before SIGKILL")
    _add_json_flag(p_stop)

    p_reload = sub.add_parser("reload-bot")
    p_reload.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_reload.add_argument("--bot-id", required=True, help="Bot id to reload")
    p_reload.add_argument("--poll-interval", type=int, default=5, help="Polling interval for started bot process")
    p_reload.add_argument("--prepare-first", action="store_true", help="Run prepare before reloading bot")
    p_reload.add_argument("--timeout-sec", type=int, default=8, help="Graceful stop timeout before SIGKILL")
    _add_restore_defaults_flag(p_reload)
    _add_json_flag(p_reload)

    p_web = sub.add_parser("web", help="Run fleet web console in foreground")
    p_web.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_web.add_argument("--host", default="127.0.0.1", help="Host to bind web console")
    p_web.add_argument("--port", type=int, default=8787, help="Port to bind web console")

    p_web_start = sub.add_parser("web-start", help="Start fleet web console in background")
    p_web_start.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_web_start.add_argument("--host", default="127.0.0.1", help="Host to bind web console")
    p_web_start.add_argument("--port", type=int, default=8787, help="Port to bind web console")
    _add_json_flag(p_web_start)

    p_web_stop = sub.add_parser("web-stop", help="Stop fleet web console")
    p_web_stop.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_web_stop.add_argument("--timeout-sec", type=int, default=8, help="Graceful stop timeout before SIGKILL")
    _add_json_flag(p_web_stop)

    p_web_status = sub.add_parser("web-status", help="Show fleet web console status")
    p_web_status.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    _add_json_flag(p_web_status)

    p_agent_system = sub.add_parser(
        "bot-agent-system",
        aliases=["bot-agent-sysyem"],
        help="Install or manage extra agent systems",
    )
    p_agent_system_sub = p_agent_system.add_subparsers(dest="agent_system_command", required=True)

    p_agent_add = p_agent_system_sub.add_parser("add", help="Install an agent system from a zip archive")
    p_agent_add.add_argument("archive", help="Path to an agent system zip archive")
    p_agent_add.add_argument("--manifest", default=str(DEFAULT_MANIFEST_PATH), help="Path to fleet manifest JSON")
    p_agent_scope = p_agent_add.add_mutually_exclusive_group()
    p_agent_scope.add_argument("--bot-id", default="", help="Install only into one bot runtime")
    p_agent_scope.add_argument("--global", dest="install_global", action="store_true", help="Install into project assets/agent_library")
    p_agent_add.add_argument("--reload-after", action="store_true", help="Reload target bot after install (requires --bot-id)")
    p_agent_add.add_argument("--poll-interval", type=int, default=5, help="Polling interval when reloading bot")
    _add_json_flag(p_agent_add)
    return parser


def main() -> int:
    parser = build_parser()
    args = parser.parse_args()
    json_output = bool(getattr(args, "json", False))
    manifest_path = Path(args.manifest).expanduser().resolve()

    if args.command in {"init", "config"}:
        if bool(args.interactive):
            seed_manifest: Dict[str, Any] = {}
            if manifest_path.exists():
                seed_manifest = _load_json(manifest_path)
            target_manifest_path, payload = _interactive_manifest_setup(
                manifest_path,
                seed_manifest=seed_manifest,
            )
            if target_manifest_path.exists() and not bool(args.force):
                should_overwrite = _prompt_bool(
                    f"manifest exists, overwrite? {target_manifest_path}",
                    False,
                )
                if not should_overwrite:
                    print("[ABORT] manifest not changed")
                    return 1
            _dump_json(target_manifest_path, payload)
            print(f"[OK] manifest saved: {target_manifest_path}")
            print("[NEXT] run: python scripts/fleet_one_click.py up --manifest", target_manifest_path)
            if bool(args.open_config):
                _open_config_file(target_manifest_path)
            return 0

        created = _scaffold_manifest(manifest_path, force=bool(args.force))
        if created:
            print(f"[OK] manifest created from template: {manifest_path}")
        else:
            print(f"[OK] manifest already exists: {manifest_path}")
        print("[NEXT] fill llm_config_path/runtime_root and bot config (credentials if needed) before running 'up'")
        if bool(args.open_config):
            _open_config_file(manifest_path)
        return 0

    if args.command == "web":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        if not FLEET_WEB_CONSOLE_PATH.exists():
            raise SystemExit(f"[ERROR] fleet web script not found: {FLEET_WEB_CONSOLE_PATH}")
        run_cmd = [
            sys.executable,
            str(FLEET_WEB_CONSOLE_PATH),
            "--manifest",
            str(manifest_path),
            "--host",
            str(args.host),
            "--port",
            str(int(args.port)),
        ]
        env = os.environ.copy()
        return _run(run_cmd, env=env, check=False)

    if args.command == "web-start":
        payload = _proc_start_fleet_web(
            manifest_path,
            host=str(args.host or "127.0.0.1"),
            port=int(args.port or 8787),
        )
        if json_output:
            print_json(payload)
        else:
            format_web_payload(payload, title="WEB")
        return 0

    if args.command == "web-stop":
        if manifest_path.exists():
            payload = _proc_stop_fleet_web(
                manifest_path,
                timeout_sec=int(args.timeout_sec),
            )
        else:
            payload = _proc_stop_fleet_web_fallback(
                timeout_sec=int(args.timeout_sec),
            )
        if json_output:
            print_json(payload)
        else:
            format_web_payload(payload, title="WEB")
        return 0

    if args.command == "web-status":
        payload = _proc_fleet_web_status_payload(manifest_path)
        if json_output:
            print_json(payload)
        else:
            format_web_payload(payload, title="WEB")
        return 0

    if args.command == "bot-agent-system":
        if args.agent_system_command == "add":
            archive_path = Path(str(args.archive)).expanduser().resolve()
            bot_id = str(args.bot_id or "").strip()
            if bool(args.reload_after) and not bot_id:
                raise SystemExit("[ERROR] --reload-after requires --bot-id")

            if bot_id:
                if not manifest_path.exists():
                    raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
                _manifest, _target, user_data_root, _llm = _proc_resolve_bot_from_manifest(manifest_path, bot_id)
                destination_root = (user_data_root / "agent_library").resolve()
                payload = _install_agent_system_archive(archive_path, destination_root)
                payload["scope"] = "bot_runtime"
                payload["bot_id"] = bot_id
                payload["next_step"] = f"cheapclaw reload-bot --bot-id {bot_id}"
                if bool(args.reload_after):
                    reload_result = _proc_stop_single_bot_from_manifest(
                        manifest_path,
                        bot_id=bot_id,
                        timeout_sec=8,
                    )
                    start_result = _proc_start_single_bot_from_manifest(
                        manifest_path,
                        bot_id=bot_id,
                        poll_interval=int(args.poll_interval),
                    )
                    payload["reload"] = {"stop": reload_result, "start": start_result}
                    payload["next_step"] = "已自动 reload 目标 bot"
            else:
                destination_root = AGENT_LIBRARY_ROOT.resolve()
                payload = _install_agent_system_archive(archive_path, destination_root)
                payload["scope"] = "global_assets"
                payload["next_step"] = "运行 `cheapclaw prepare` 或 `cheapclaw reload-bot --bot-id <id> --prepare-first` 使 bot 生效"

            if json_output:
                print_json(payload)
            else:
                format_agent_system_payload(payload)
            return 0

    if args.command == "list-bots":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        bots = _list_manifest_bots(manifest_path, enabled_only=False)
        payload = {
            "status": "success",
            "manifest_path": str(manifest_path),
            "bot_count": len(bots),
            "bots": [
                {
                    "bot_id": str(item.get("bot_id") or ""),
                    "display_name": str(item.get("display_name") or item.get("bot_id") or ""),
                    "channel": str(item.get("channel") or ""),
                    "enabled": bool(item.get("enabled", True)),
                    "serve_webhooks": bool(item.get("serve_webhooks", False)),
                }
                for item in bots
            ],
        }
        if json_output:
            print_json(payload)
        else:
            format_list_bots_payload(payload)
        return 0

    if args.command == "status":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        bot_id = str(args.bot_id or "").strip()
        if bot_id:
            payload = {
                "status": "success",
                "manifest_path": str(manifest_path),
                "bot_count": 1,
                "running_count": 1 if _proc_status_single_bot_from_manifest(manifest_path, bot_id).get("alive") else 0,
                "bots": [_proc_status_single_bot_from_manifest(manifest_path, bot_id)],
            }
        else:
            payload = _proc_status_all_bots_from_manifest(manifest_path)
        if json_output:
            payload = {
                **payload,
                "web_console": _proc_fleet_web_status_payload(manifest_path),
            }
            print_json(payload)
        else:
            format_status_payload(payload)
            format_web_payload(_proc_fleet_web_status_payload(manifest_path), title="WEB")
        return 0

    if args.command == "start":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        if bool(args.prepare_first):
            result = prepare_from_manifest(
                manifest_path,
                write_fleet_to=None,
                run_capture=_run_capture,
                restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
            )
            if json_output:
                print_json(result)
            else:
                format_prepare_payload(result)
        bot_id = str(args.bot_id or "").strip()
        if bot_id:
            payload = _proc_start_single_bot_from_manifest(
                manifest_path,
                bot_id=bot_id,
                poll_interval=int(args.poll_interval),
            )
        else:
            payload = _proc_start_all_bots_from_manifest(
                manifest_path,
                poll_interval=int(args.poll_interval),
            )
        if json_output:
            print_json(payload)
        else:
            format_start_stop_payload(payload, title="START")
        if bool(args.with_web_console):
            web_payload = _proc_start_fleet_web(
                manifest_path,
                host=str(args.web_host or "127.0.0.1"),
                port=int(args.web_port or 8787),
            )
            if json_output:
                print_json(web_payload)
            else:
                format_web_payload(web_payload, title="WEB")
        return 0

    if args.command == "stop":
        if not manifest_path.exists():
            bot_id = str(args.bot_id or "").strip()
            if bot_id:
                raise SystemExit(
                    f"[ERROR] manifest not found: {manifest_path}\n"
                    f"[HINT] cannot stop specific bot '{bot_id}' without manifest; "
                    "use `cheapclaw stop` to stop all by fallback scan."
                )
            payload = _proc_stop_all_fallback_without_manifest(
                timeout_sec=int(args.timeout_sec),
            )
            if json_output:
                print_json(payload)
            else:
                format_emergency_stop_payload(payload)
            return 0
        bot_id = str(args.bot_id or "").strip()
        if bot_id:
            payload = _proc_stop_single_bot_from_manifest(
                manifest_path,
                bot_id=bot_id,
                timeout_sec=int(args.timeout_sec),
            )
        else:
            payload = _proc_stop_all_bots_from_manifest(
                manifest_path,
                timeout_sec=int(args.timeout_sec),
            )
        if json_output:
            print_json(payload)
        else:
            format_start_stop_payload(payload, title="STOP")
        if bool(args.with_web_console):
            web_payload = _proc_stop_fleet_web(
                manifest_path,
                timeout_sec=int(args.timeout_sec),
            )
            if json_output:
                print_json(web_payload)
            else:
                format_web_payload(web_payload, title="WEB")
        return 0

    if args.command == "restart":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        if bool(args.prepare_first):
            result = prepare_from_manifest(
                manifest_path,
                write_fleet_to=None,
                run_capture=_run_capture,
                restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
            )
            if json_output:
                print_json(result)
            else:
                format_prepare_payload(result)
        bot_id = str(args.bot_id or "").strip()
        if bot_id:
            stop_payload = _proc_stop_single_bot_from_manifest(
                manifest_path,
                bot_id=bot_id,
                timeout_sec=int(args.timeout_sec),
            )
            start_payload = _proc_start_single_bot_from_manifest(
                manifest_path,
                bot_id=bot_id,
                poll_interval=int(args.poll_interval),
            )
        else:
            stop_payload = _proc_stop_all_bots_from_manifest(
                manifest_path,
                timeout_sec=int(args.timeout_sec),
            )
            start_payload = _proc_start_all_bots_from_manifest(
                manifest_path,
                poll_interval=int(args.poll_interval),
            )
        payload = {
            "status": "success",
            "manifest_path": str(manifest_path),
            "stop": stop_payload,
            "start": start_payload,
        }
        if json_output:
            print_json(payload)
        else:
            format_restart_payload(payload)
        if bool(args.with_web_console):
            web_payload = _proc_start_fleet_web(
                manifest_path,
                host=str(args.web_host or "127.0.0.1"),
                port=int(args.web_port or 8787),
            )
            if json_output:
                print_json(web_payload)
            else:
                format_web_payload(web_payload, title="WEB")
        return 0

    if args.command == "logs":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        status = _proc_status_single_bot_from_manifest(manifest_path, str(args.bot_id))
        log_key = "loop_log_path" if str(args.kind) == "loop" else "service_log_path"
        log_path = Path(str(status.get(log_key) or "")).expanduser().resolve()
        _print_log_tail(log_path, lines=int(args.lines))
        return 0

    if args.command == "add-bot":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}\n[HINT] run `cheapclaw config` first.")
        new_bot_id = _interactive_add_bot(manifest_path)
        print(f"[OK] bot added to manifest: {new_bot_id}")
        if bool(args.prepare_after):
            result = prepare_from_manifest(
                manifest_path,
                write_fleet_to=None,
                run_capture=_run_capture,
                restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
            )
            if json_output:
                print_json(result)
            else:
                format_prepare_payload(result)
        if bool(args.start_after):
            start_result = _proc_start_single_bot_from_manifest(
                manifest_path,
                bot_id=new_bot_id,
                poll_interval=int(args.poll_interval),
            )
            if json_output:
                print_json(start_result)
            else:
                format_start_stop_payload(start_result, title="START")
        return 0

    if args.command == "start-bot":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        if bool(args.prepare_first):
            result = prepare_from_manifest(
                manifest_path,
                write_fleet_to=None,
                run_capture=_run_capture,
                restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
            )
            if json_output:
                print_json(result)
            else:
                format_prepare_payload(result)
        start_result = _proc_start_single_bot_from_manifest(
            manifest_path,
            bot_id=str(args.bot_id),
            poll_interval=int(args.poll_interval),
        )
        if json_output:
            print_json(start_result)
        else:
            format_start_stop_payload(start_result, title="START")
        return 0

    if args.command == "stop-bot":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        stop_result = _proc_stop_single_bot_from_manifest(
            manifest_path,
            bot_id=str(args.bot_id),
            timeout_sec=int(args.timeout_sec),
        )
        if json_output:
            print_json(stop_result)
        else:
            format_start_stop_payload(stop_result, title="STOP")
        return 0

    if args.command == "reload-bot":
        if not manifest_path.exists():
            raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
        if bool(args.prepare_first):
            result = prepare_from_manifest(
                manifest_path,
                write_fleet_to=None,
                run_capture=_run_capture,
                restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
            )
            if json_output:
                print_json(result)
            else:
                format_prepare_payload(result)
        stop_result = _proc_stop_single_bot_from_manifest(
            manifest_path,
            bot_id=str(args.bot_id),
            timeout_sec=int(args.timeout_sec),
        )
        if json_output:
            print_json(stop_result)
        else:
            format_start_stop_payload(stop_result, title="STOP")
        start_result = _proc_start_single_bot_from_manifest(
            manifest_path,
            bot_id=str(args.bot_id),
            poll_interval=int(args.poll_interval),
        )
        if json_output:
            print_json(start_result)
        else:
            format_start_stop_payload(start_result, title="START")
        return 0

    if not manifest_path.exists():
        if bool(args.interactive):
            seed_manifest: Dict[str, Any] = {}
            target_manifest_path, payload = _interactive_manifest_setup(
                manifest_path,
                seed_manifest=seed_manifest,
            )
            _dump_json(target_manifest_path, payload)
            manifest_path = target_manifest_path
            print(f"[OK] manifest saved: {manifest_path}")
            if bool(args.open_config_on_init):
                _open_config_file(manifest_path)
        else:
            if not bool(args.init_if_missing):
                raise SystemExit(f"[ERROR] manifest not found: {manifest_path}")
            _scaffold_manifest(manifest_path, force=False)
            print(f"[INIT] manifest auto-created from template: {manifest_path}")
            print("[NEXT] edit manifest first, then rerun the same command.")
            if bool(args.open_config_on_init):
                _open_config_file(manifest_path)
            return 0

    manifest_preview = _load_json(manifest_path)
    placeholder_fields = _scan_placeholder_fields(manifest_preview)
    if placeholder_fields and bool(args.interactive):
        print("[SETUP] placeholder values detected, entering interactive setup")
        target_manifest_path, payload = _interactive_manifest_setup(
            manifest_path,
            seed_manifest=manifest_preview,
        )
        _dump_json(target_manifest_path, payload)
        manifest_path = target_manifest_path
        manifest_preview = payload
        placeholder_fields = _scan_placeholder_fields(manifest_preview)
    if placeholder_fields:
        print("[ERROR] manifest still contains placeholder values:")
        for item in placeholder_fields[:20]:
            print(" -", item)
        if len(placeholder_fields) > 20:
            print(f" - ... and {len(placeholder_fields) - 20} more")
        print(f"[HINT] Please edit: {manifest_path}")
        if bool(args.open_config_on_init):
            _open_config_file(manifest_path)
        return 2

    fleet_out = Path(args.fleet_config_out).expanduser().resolve() if str(args.fleet_config_out).strip() else None
    result = prepare_from_manifest(
        manifest_path,
        write_fleet_to=fleet_out,
        run_capture=_run_capture,
        restore_package_defaults=bool(getattr(args, "restore_package_defaults", False)),
    )
    if json_output:
        print_json(result)
    else:
        format_prepare_payload(result)

    if args.command == "prepare":
        print("[OK] prepare done")
        return 0

    if args.command in {"up", "run"} and not bool(args.foreground):
        payload = _proc_start_all_bots_from_manifest(
            manifest_path,
            poll_interval=int(args.poll_interval),
        )
        if json_output:
            print_json(payload)
        else:
            format_start_stop_payload(payload, title="START")
        if bool(args.with_web_console):
            web_payload = _proc_start_fleet_web(
                manifest_path,
                host=str(args.web_host or "127.0.0.1"),
                port=int(args.web_port or 8787),
            )
            if json_output:
                print_json(web_payload)
            else:
                format_web_payload(web_payload, title="WEB")
        print("[OK] services are running in background; use `cheapclaw status` and `cheapclaw logs --bot-id <id>`")
        return 0

    run_cmd = [
        sys.executable,
        str(SERVICE_PATH),
        "--fleet-config-path",
        str(result["fleet_config_path"]),
        "--run-fleet",
        "--poll-interval",
        str(max(1, int(args.poll_interval))),
    ]
    manifest = _load_json(manifest_path)
    env = _child_process_env(manifest)
    return _run(run_cmd, env=env, check=False)


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
from pathlib import Path
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import cancel_plan, cancel_plans
from tool_server_lite.tools.file_tools import BaseTool

class CheapClawCancelPlanTool(BaseTool):
    name = "cheapclaw_cancel_plan"
    def execute(self, task_id, parameters):
        plan_id = str(parameters.get("plan_id") or "").strip()
        if plan_id:
            ok = cancel_plan(plan_id)
            if not ok:
                return {"status": "error", "output": "", "error": f"plan not found: {plan_id}"}
            return {
                "status": "success",
                "output": f"cancelled {plan_id}",
                "matched_plan_ids": [plan_id],
                "cancelled_plan_ids": [plan_id],
                "matched_count": 1,
                "cancelled_count": 1,
            }
        result = cancel_plans(
            name=str(parameters.get("name") or "").strip(),
            task_id=str(parameters.get("task_id") or "").strip(),
            channel=str(parameters.get("channel") or "").strip(),
            conversation_id=str(parameters.get("conversation_id") or "").strip(),
            scope=str(parameters.get("scope") or "").strip(),
        )
        if result.get("status") != "success":
            return {"status": "error", "output": "", "error": str(result.get("error") or "plan not found")}
        matched_ids = result.get("matched_plan_ids") or []
        cancelled_ids = result.get("cancelled_plan_ids") or []
        return {
            "status": "success",
            "output": f"matched={len(matched_ids)} cancelled={len(cancelled_ids)}",
            **result,
        }



#!/usr/bin/env python3
from pathlib import Path
from infiagent import infiagent
from tool_server_lite.tools.file_tools import BaseTool
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import get_user_data_root, inspect_agent_system_blocked_tools

class CheapClawListAgentSystemsTool(BaseTool):
    name = "cheapclaw_list_agent_systems"
    def execute(self, task_id, parameters):
        user_root = get_user_data_root()
        agent_root = user_root / "agent_library"
        visible_names = set()
        if agent_root.exists():
            visible_names = {item.name for item in agent_root.iterdir() if item.is_dir()}
        agent = infiagent(user_data_root=str(user_root), seed_builtin_resources=False)
        payload = agent.list_agent_systems()
        items = payload.get("agent_systems")
        if not isinstance(items, list):
            items = []
        filtered = []
        blocked = []
        for item in items:
            if not isinstance(item, dict):
                continue
            name = str(item.get("name") or "").strip()
            if name.endswith("_tmp"):
                continue
            if not name or name not in visible_names:
                continue
            inspection = inspect_agent_system_blocked_tools(name, user_data_root=user_root)
            if inspection.get("blocked"):
                blocked.append({
                    "name": name,
                    "blocked_tool_names": list(inspection.get("blocked_tool_names") or []),
                    "hits": list(inspection.get("hits") or []),
                })
                continue
            filtered.append(item)
        payload["agent_systems"] = filtered
        payload["runtime_agent_library_dir"] = str(agent_root)
        payload["visible_agent_system_names"] = [
            str(item.get("name") or "") for item in payload["agent_systems"] if str(item.get("name") or "")
        ]
        payload["blocked_agent_systems"] = blocked
        payload["recommended_defaults"] = {
            "CheapClawSupervisor": "supervisor_agent",
            "CheapClawWorkerGeneral": "worker_agent",
        }
        return payload



#!/usr/bin/env python3
from __future__ import annotations

import json
import re
import uuid
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple

try:
    import fcntl
except ImportError:  # pragma: no cover
    fcntl = None

from cheapclaw_channel_base import ChannelAdapter
from cheapclaw_runtime_state import atomic_write_text, load_json
from tool_runtime_helpers import list_peer_bots, now_iso


class LocalWebAdapter(ChannelAdapter):
    name = "localweb"
    MENTION_PATTERN = re.compile(r"@([A-Za-z0-9_.-]{1,80})")

    def __init__(self, config: Dict[str, Any], service: Any, *, logger=None):
        super().__init__(config, service, logger=logger)
        self.bot_id = str(config.get("bot_id") or service.bot_id).strip() or service.bot_id
        self.display_name = str(config.get("display_name") or service.bot_display_name or self.bot_id).strip() or self.bot_id
        self.require_mention_in_group = bool(config.get("require_mention_in_group", True))
        self.shared_root = self._resolve_shared_root()
        self.shared_root.mkdir(parents=True, exist_ok=True)
        self.events_path = self.shared_root / "events.jsonl"
        self.events_lock_path = self.shared_root / "events.lock"
        self.conversations_path = self.shared_root / "conversations.json"
        self._ensure_storage()

    def _resolve_shared_root(self) -> Path:
        if self.service.fleet_config_path:
            fleet_path = Path(self.service.fleet_config_path).expanduser().resolve()
            return fleet_path.parent / "localweb"
        return self.service.paths.user_data_root / "cheapclaw" / "localweb"

    def _ensure_storage(self) -> None:
        if not self.conversations_path.exists():
            atomic_write_text(self.conversations_path, json.dumps({"version": 1, "conversations": {}}, ensure_ascii=False, indent=2))
        self.events_path.touch(exist_ok=True)
        self.events_lock_path.touch(exist_ok=True)

    def _load_conversations_payload(self) -> Dict[str, Any]:
        payload = load_json(self.conversations_path, {"version": 1, "conversations": {}})
        if not isinstance(payload, dict):
            payload = {"version": 1, "conversations": {}}
        payload.setdefault("version", 1)
        conversations = payload.get("conversations")
        if not isinstance(conversations, dict):
            conversations = {}
        normalized: Dict[str, Dict[str, Any]] = {}
        for raw_id, item in conversations.items():
            conversation_id = str(raw_id or "").strip()
            if not conversation_id or not isinstance(item, dict):
                continue
            participant_bots = []
            for bot_id in item.get("participant_bots", []):
                bot_text = str(bot_id or "").strip()
                if bot_text and bot_text not in participant_bots:
                    participant_bots.append(bot_text)
            normalized[conversation_id] = {
                "conversation_id": conversation_id,
                "display_name": str(item.get("display_name") or conversation_id).strip() or conversation_id,
                "conversation_type": "group" if str(item.get("conversation_type") or "group").strip() == "group" else "person",
                "participant_bots": participant_bots,
                "require_mention": bool(item.get("require_mention", self.require_mention_in_group)),
                "created_at": str(item.get("created_at") or now_iso()),
                "updated_at": str(item.get("updated_at") or now_iso()),
                "created_by": str(item.get("created_by") or "system"),
            }
        payload["conversations"] = normalized
        return payload

    def _save_conversations_payload(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        atomic_write_text(self.conversations_path, json.dumps(payload, ensure_ascii=False, indent=2))
        return payload

    def list_conversations(self) -> List[Dict[str, Any]]:
        payload = self._load_conversations_payload()
        items = [item for item in payload.get("conversations", {}).values() if isinstance(item, dict)]
        items.sort(key=lambda item: str(item.get("updated_at") or ""), reverse=True)
        return items

    def ensure_conversation_record(
        self,
        *,
        conversation_id: str,
        display_name: str = "",
        conversation_type: str = "",
        participant_bots: Optional[List[str]] = None,
        require_mention: Optional[bool] = None,
        created_by: str = "",
        include_self_participant: bool = True,
    ) -> Dict[str, Any]:
        conv_id = str(conversation_id or "").strip()
        if not conv_id:
            conv_id = f"lc_{uuid.uuid4().hex[:12]}"
        payload = self._load_conversations_payload()
        conversations = payload.setdefault("conversations", {})
        existing = conversations.get(conv_id) if isinstance(conversations.get(conv_id), dict) else {}
        if not existing:
            existing = {
                "conversation_id": conv_id,
                "display_name": conv_id,
                "conversation_type": "group",
                "participant_bots": [],
                "require_mention": self.require_mention_in_group,
                "created_at": now_iso(),
                "updated_at": now_iso(),
                "created_by": created_by or "system",
            }
        if display_name.strip():
            existing["display_name"] = display_name.strip()
        if conversation_type.strip() in {"group", "person"}:
            existing["conversation_type"] = conversation_type.strip()
        if require_mention is not None:
            existing["require_mention"] = bool(require_mention)
        members = list(existing.get("participant_bots") or [])
        if participant_bots is not None:
            members = []
            for bot_id in participant_bots:
                bot_text = str(bot_id or "").strip()
                if bot_text and bot_text not in members:
                    members.append(bot_text)
        if include_self_participant and self.bot_id not in members:
            members.append(self.bot_id)
        existing["participant_bots"] = members
        existing["updated_at"] = now_iso()
        conversations[conv_id] = existing
        self._save_conversations_payload(payload)
        return existing

    def _known_bot_id_map(self) -> Dict[str, str]:
        items = list_peer_bots(current_bot_id=self.service.bot_id, include_self=True, path=self.service.fleet_config_path)
        if not items:
            items = [{"bot_id": self.service.bot_id, "display_name": self.service.bot_display_name}]
        mapping: Dict[str, str] = {}
        for item in items:
            bot_id = str(item.get("bot_id") or "").strip()
            display_name = str(item.get("display_name") or "").strip()
            if bot_id:
                mapping[bot_id.lower()] = bot_id
            if display_name:
                mapping[display_name.lower()] = bot_id or display_name
        return mapping

    def parse_mentions(self, text: str) -> List[str]:
        bot_map = self._known_bot_id_map()
        mentions: List[str] = []
        for token in self.MENTION_PATTERN.findall(str(text or "")):
            key = str(token or "").strip().lower()
            if not key:
                continue
            resolved = bot_map.get(key) or str(token or "").strip()
            if resolved and resolved not in mentions:
                mentions.append(resolved)
        return mentions

    def _append_event_line(self, payload: Dict[str, Any]) -> None:
        lock_file = open(self.events_lock_path, "a+", encoding="utf-8")
        try:
            if fcntl is not None:
                fcntl.flock(lock_file.fileno(), fcntl.LOCK_EX)
            with open(self.events_path, "a", encoding="utf-8") as fh:
                fh.write(json.dumps(payload, ensure_ascii=False) + "\n")
        finally:
            if fcntl is not None:
                fcntl.flock(lock_file.fileno(), fcntl.LOCK_UN)
            lock_file.close()

    def emit_event(
        self,
        *,
        conversation_id: str,
        message: str,
        sender_id: str,
        sender_name: str,
        sender_type: str,
        attachments: Optional[List[Dict[str, Any]]] = None,
        display_name: str = "",
        participant_bots: Optional[List[str]] = None,
        require_mention: Optional[bool] = None,
    ) -> Dict[str, Any]:
        text = str(message or "").strip()
        if not text and not (attachments or []):
            return {"status": "error", "error": "localweb message or attachments are required"}
        mentions = self.parse_mentions(text)
        merged_participants = list(participant_bots or [])
        for mention in mentions:
            if mention not in merged_participants:
                merged_participants.append(mention)
        conv = self.ensure_conversation_record(
            conversation_id=conversation_id,
            display_name=display_name,
            conversation_type="group" if len(merged_participants) > 1 else "person",
            participant_bots=merged_participants if merged_participants else None,
            require_mention=require_mention,
            created_by=sender_id,
        )
        event_payload = {
            "event_id": f"lwevt_{uuid.uuid4().hex[:12]}",
            "channel": "localweb",
            "conversation_id": str(conv.get("conversation_id") or ""),
            "conversation_type": str(conv.get("conversation_type") or "group"),
            "display_name": str(conv.get("display_name") or conv.get("conversation_id") or ""),
            "participant_bots": list(conv.get("participant_bots") or []),
            "require_mention": bool(conv.get("require_mention", self.require_mention_in_group)),
            "sender_id": str(sender_id or ""),
            "sender_name": str(sender_name or sender_id or ""),
            "sender_type": str(sender_type or "user"),
            "message_text": text,
            "attachments": attachments or [],
            "mentions": mentions,
            "timestamp": now_iso(),
        }
        self._append_event_line(event_payload)
        return {"status": "success", "event": event_payload}

    def _load_new_bus_events(self) -> List[Tuple[int, Dict[str, Any]]]:
        state = self.service.load_runtime_state()
        offsets = state.setdefault("localweb_offsets", {})
        line_offset = int(offsets.get("line") or 0)
        lines = self.events_path.read_text(encoding="utf-8", errors="ignore").splitlines()
        new_events: List[Tuple[int, Dict[str, Any]]] = []
        next_offset = line_offset
        total_lines = len(lines)
        for idx, line in enumerate(lines[line_offset:], start=line_offset + 1):
            if not line.strip():
                next_offset = idx
                continue
            try:
                payload = json.loads(line)
            except Exception:
                if idx == total_lines:
                    break
                next_offset = idx
                continue
            if isinstance(payload, dict):
                new_events.append((idx, payload))
            next_offset = idx
        offsets["line"] = next_offset
        state["localweb_offsets"] = offsets
        self.service.save_runtime_state(state)
        return new_events

    def _should_deliver(self, payload: Dict[str, Any]) -> Tuple[bool, bool, Dict[str, Any]]:
        conversation_id = str(payload.get("conversation_id") or "").strip()
        if not conversation_id:
            return False, False, {}
        conv = self.ensure_conversation_record(
            conversation_id=conversation_id,
            display_name=str(payload.get("display_name") or ""),
            conversation_type=str(payload.get("conversation_type") or ""),
            participant_bots=list(payload.get("participant_bots") or []),
            require_mention=payload.get("require_mention"),
            include_self_participant=False,
        )
        sender_type = str(payload.get("sender_type") or "user").strip().lower() or "user"
        sender_id = str(payload.get("sender_id") or "").strip()
        mentions = [str(item).strip() for item in payload.get("mentions", []) if str(item).strip()]
        mention_to_bot = self.bot_id in mentions
        if sender_type == "bot":
            if sender_id == self.bot_id:
                return False, False, conv
            if not mention_to_bot:
                return False, False, conv
        if mentions and not mention_to_bot:
            return False, False, conv
        participants = [str(item).strip() for item in conv.get("participant_bots", []) if str(item).strip()]
        if participants and self.bot_id not in participants and not mention_to_bot:
            return False, mention_to_bot, conv
        conversation_type = str(conv.get("conversation_type") or "group")
        require_mention = bool(conv.get("require_mention", self.require_mention_in_group))
        is_group = conversation_type == "group"
        if is_group and require_mention and not mention_to_bot:
            return False, mention_to_bot, conv
        return True, mention_to_bot or (not (is_group and require_mention)), conv

    def poll_events(self) -> List[Dict[str, Any]]:
        events: List[Dict[str, Any]] = []
        for index, payload in self._load_new_bus_events():
            deliver, is_mention_to_bot, conv = self._should_deliver(payload)
            if not deliver:
                continue
            sender_name = str(payload.get("sender_name") or payload.get("sender_id") or "").strip()
            sender_id = str(payload.get("sender_id") or "").strip()
            event_id = str(payload.get("event_id") or f"lwevt_{index}")
            events.append(
                {
                    "event_id": f"{event_id}:{self.bot_id}",
                    "channel": "localweb",
                    "conversation_id": str(conv.get("conversation_id") or payload.get("conversation_id") or ""),
                    "conversation_type": str(conv.get("conversation_type") or payload.get("conversation_type") or "group"),
                    "display_name": str(conv.get("display_name") or payload.get("display_name") or payload.get("conversation_id") or ""),
                    "sender_id": sender_id,
                    "sender_name": sender_name or "localweb_user",
                    "message_id": event_id,
                    "message_text": str(payload.get("message_text") or ""),
                    "attachments": payload.get("attachments") or [],
                    "timestamp": str(payload.get("timestamp") or now_iso()),
                    "is_mention_to_bot": bool(is_mention_to_bot),
                    "require_mention": bool(conv.get("require_mention", self.require_mention_in_group)),
                }
            )
        return events

    def send_message(self, conversation_id: str, message: str, attachments: Optional[List[Dict[str, Any]]] = None) -> Tuple[bool, str]:
        result = self.emit_event(
            conversation_id=conversation_id,
            message=message,
            sender_id=self.service.bot_id,
            sender_name=self.service.bot_display_name,
            sender_type="bot",
            attachments=attachments or [],
        )
        if result.get("status") != "success":
            return False, str(result.get("error") or "localweb send failed")
        event = result.get("event") if isinstance(result.get("event"), dict) else {}
        return True, str(event.get("event_id") or "")



#!/usr/bin/env python3
from pathlib import Path
from infiagent import infiagent
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import assert_managed_task_id
from tool_server_lite.tools.file_tools import BaseTool

class CheapClawResetTaskTool(BaseTool):
    name = "cheapclaw_reset_task"
    def execute(self, task_id, parameters):
        try:
            target_task_id = assert_managed_task_id(str(Path(parameters.get("task_id") or "").expanduser().resolve()))
        except ValueError as exc:
            return {"status": "error", "output": "", "error": str(exc)}
        if not target_task_id:
            return {"status": "error", "output": "", "error": "task_id is required"}
        agent = infiagent(user_data_root=str(Path(__import__('os').environ.get('MLA_USER_DATA_ROOT','~/mla_v3')).expanduser().resolve()))
        return agent.reset_task(
            task_id=target_task_id,
            preserve_history=bool(parameters.get("preserve_history", True)),
            kill_background_processes=bool(parameters.get("kill_background_processes", True)),
            reason=str(parameters.get("reason") or "manual reset"),
        )



#!/usr/bin/env python3
from __future__ import annotations

import os
from pathlib import Path
from typing import Any, Dict, List, Tuple

try:
    from .fleet_manifest import (
        DEFAULT_RUNTIME_ROOT,
        _bot_seed_is_placeholder,
        _dump_json,
        _load_json,
        _normalize_context_settings,
        _normalize_model_name,
        _write_minimal_llm_config,
    )
except ImportError:
    from fleet_manifest import (
        DEFAULT_RUNTIME_ROOT,
        _bot_seed_is_placeholder,
        _dump_json,
        _load_json,
        _normalize_context_settings,
        _normalize_model_name,
        _write_minimal_llm_config,
    )


def _prompt_text(label: str, default: str = "", *, required: bool = False) -> str:
    while True:
        hint = f" [{default}]" if default else ""
        raw = input(f"{label}{hint}: ").strip()
        if raw:
            return raw
        if default:
            return default
        if not required:
            return ""
        print("[INPUT] this field is required")


def _prompt_bool(label: str, default: bool) -> bool:
    suffix = "Y/n" if default else "y/N"
    while True:
        raw = input(f"{label} [{suffix}]: ").strip().lower()
        if not raw:
            return default
        if raw in {"y", "yes"}:
            return True
        if raw in {"n", "no"}:
            return False
        print("[INPUT] please answer y or n")


def _prompt_int(label: str, default: int, *, min_value: int = 0) -> int:
    while True:
        raw = input(f"{label} [{default}]: ").strip()
        if not raw:
            return max(min_value, default)
        try:
            value = int(raw)
        except ValueError:
            print("[INPUT] please enter a valid integer")
            continue
        if value < min_value:
            print(f"[INPUT] value must be >= {min_value}")
            continue
        return value


def _prompt_channel(default: str = "localweb") -> str:
    default_norm = default if default in {"telegram", "feishu", "whatsapp", "discord", "qq", "wechat", "localweb"} else "localweb"
    mapping = {
        "1": "telegram",
        "2": "feishu",
        "3": "whatsapp",
        "4": "discord",
        "5": "qq",
        "6": "wechat",
        "7": "localweb",
        "telegram": "telegram",
        "feishu": "feishu",
        "whatsapp": "whatsapp",
        "discord": "discord",
        "qq": "qq",
        "wechat": "wechat",
        "localweb": "localweb",
    }
    while True:
        raw = input(
            f"channel [1=telegram, 2=feishu, 3=whatsapp, 4=discord, 5=qq, 6=wechat, 7=localweb] [{default_norm}]: "
        ).strip().lower()
        if not raw:
            return default_norm
        value = mapping.get(raw)
        if value:
            return value
        print("[INPUT] channel must be one of: 1/2/3/4/5/6/7 or telegram/feishu/whatsapp/discord/qq/wechat/localweb")


def _prompt_bot_payload(
    *,
    index: int,
    seed_bot: Dict[str, Any] | None = None,
    existing_bot_ids: set[str] | None = None,
) -> Dict[str, Any]:
    seed = seed_bot if isinstance(seed_bot, dict) else {}
    existing = existing_bot_ids if existing_bot_ids is not None else set()
    seed_channel = str(seed.get("channel") or "localweb").strip().lower()
    print(f"\n[SETUP] bot {index}")
    channel = _prompt_channel(seed_channel)

    while True:
        bot_id_default = str(seed.get("bot_id") or f"{channel}_bot_{index}")
        bot_id = _prompt_text("bot_id", bot_id_default, required=True)
        if bot_id in existing:
            print(f"[INPUT] bot_id already exists: {bot_id}")
            continue
        break

    display_name = _prompt_text("display_name", str(seed.get("display_name") or bot_id), required=True)
    enabled = _prompt_bool("enabled", bool(seed.get("enabled", True)))
    default_webhooks = bool(seed.get("serve_webhooks", channel in {"localweb", "qq", "wechat"}))
    serve_webhooks = _prompt_bool("serve_webhooks", default_webhooks)

    bot_payload: Dict[str, Any] = {
        "bot_id": bot_id,
        "display_name": display_name,
        "channel": channel,
        "enabled": enabled,
        "serve_webhooks": serve_webhooks,
    }
    if serve_webhooks:
        bot_payload["host"] = _prompt_text("host", str(seed.get("host") or "127.0.0.1"), required=True)
        bot_payload["port"] = _prompt_int("port", int(seed.get("port") or (8764 + index)), min_value=1)

    if channel == "telegram":
        seed_telegram = seed.get("telegram") if isinstance(seed.get("telegram"), dict) else {}
        token = _prompt_text("telegram.bot_token", str(seed_telegram.get("bot_token") or ""), required=True)
        chats_default = ",".join(str(x) for x in seed_telegram.get("allowed_chats", []) if str(x).strip())
        chats_raw = _prompt_text("telegram.allowed_chats (comma-separated)", chats_default)
        bot_payload["telegram"] = {"bot_token": token, "allowed_chats": [x.strip() for x in chats_raw.split(",") if x.strip()]}
    elif channel == "feishu":
        seed_feishu = seed.get("feishu") if isinstance(seed.get("feishu"), dict) else {}
        bot_payload["feishu"] = {
            "mode": _prompt_text("feishu.mode", str(seed_feishu.get("mode") or "long_connection"), required=True),
            "app_id": _prompt_text("feishu.app_id", str(seed_feishu.get("app_id") or ""), required=True),
            "app_secret": _prompt_text("feishu.app_secret", str(seed_feishu.get("app_secret") or ""), required=True),
            "verify_token": _prompt_text("feishu.verify_token", str(seed_feishu.get("verify_token") or "")),
            "encrypt_key": _prompt_text("feishu.encrypt_key", str(seed_feishu.get("encrypt_key") or "")),
        }
    elif channel == "whatsapp":
        seed_wa = seed.get("whatsapp") if isinstance(seed.get("whatsapp"), dict) else {}
        bot_payload["whatsapp"] = {
            "access_token": _prompt_text("whatsapp.access_token", str(seed_wa.get("access_token") or ""), required=True),
            "phone_number_id": _prompt_text("whatsapp.phone_number_id", str(seed_wa.get("phone_number_id") or ""), required=True),
            "verify_token": _prompt_text("whatsapp.verify_token", str(seed_wa.get("verify_token") or ""), required=True),
            "api_version": _prompt_text("whatsapp.api_version", str(seed_wa.get("api_version") or "v21.0"), required=True),
        }
    elif channel == "discord":
        seed_dc = seed.get("discord") if isinstance(seed.get("discord"), dict) else {}
        bot_payload["discord"] = {
            "bot_token": _prompt_text("discord.bot_token", str(seed_dc.get("bot_token") or ""), required=True),
            "intents": _prompt_int("discord.intents", int(seed_dc.get("intents") or 37377), min_value=1),
            "require_mention_in_guild": _prompt_bool("discord.require_mention_in_guild", bool(seed_dc.get("require_mention_in_guild", True))),
        }
    elif channel in {"qq", "wechat"}:
        seed_bridge = seed.get(channel) if isinstance(seed.get(channel), dict) else {}
        bot_payload[channel] = {
            "onebot_api_base": _prompt_text(
                f"{channel}.onebot_api_base",
                str(seed_bridge.get("onebot_api_base") or f"http://127.0.0.1:{5700 if channel == 'qq' else 5701}"),
                required=True,
            ),
            "onebot_access_token": _prompt_text(f"{channel}.onebot_access_token", str(seed_bridge.get("onebot_access_token") or "")),
            "onebot_post_secret": _prompt_text(f"{channel}.onebot_post_secret", str(seed_bridge.get("onebot_post_secret") or "")),
            "onebot_self_id": _prompt_text(f"{channel}.onebot_self_id", str(seed_bridge.get("onebot_self_id") or "")),
            "require_mention_in_group": _prompt_bool(f"{channel}.require_mention_in_group", bool(seed_bridge.get("require_mention_in_group", True))),
        }
    else:
        seed_local = seed.get("localweb") if isinstance(seed.get("localweb"), dict) else {}
        bot_payload["localweb"] = {"require_mention_in_group": _prompt_bool("localweb.require_mention_in_group", bool(seed_local.get("require_mention_in_group", True)))}
    return bot_payload


def interactive_manifest_setup(default_manifest_path: Path, *, seed_manifest: Dict[str, Any] | None = None) -> Tuple[Path, Dict[str, Any]]:
    seed = seed_manifest if isinstance(seed_manifest, dict) else {}
    seed_bots = seed.get("bots")
    bots_seed: List[Dict[str, Any]] = [b for b in seed_bots if isinstance(b, dict)] if isinstance(seed_bots, list) else []
    if bots_seed and all(_bot_seed_is_placeholder(item) for item in bots_seed):
        bots_seed = []
    print("[SETUP] Interactive manifest setup")
    manifest_input = _prompt_text("manifest path", str(default_manifest_path))
    manifest_path = Path(manifest_input).expanduser().resolve()

    runtime_seed = str(seed.get("runtime_root") or "")
    runtime_default = str(DEFAULT_RUNTIME_ROOT) if _is_placeholder_text(runtime_seed) else (runtime_seed or str(DEFAULT_RUNTIME_ROOT))
    fleet_seed = str(seed.get("fleet_config_path") or "")
    fleet_default_seed = "" if _is_placeholder_text(fleet_seed) else fleet_seed
    fleet_default = fleet_default_seed or str((Path(runtime_default).expanduser().resolve() / "fleet.generated.json"))
    runtime_root = _prompt_text("runtime_root", runtime_default, required=True)
    fleet_config_path = _prompt_text("fleet_config_path", fleet_default or str((Path(runtime_root).expanduser().resolve() / "fleet.generated.json")), required=True)

    seed_proxy = seed.get("proxy_env") if isinstance(seed.get("proxy_env"), dict) else {}
    print("[SETUP] proxy_env (leave empty if not needed)")
    http_proxy = _prompt_text("proxy_env.http_proxy", str(seed_proxy.get("http_proxy") or ""))
    https_proxy = _prompt_text("proxy_env.https_proxy", str(seed_proxy.get("https_proxy") or ""))
    all_proxy = _prompt_text("proxy_env.all_proxy", str(seed_proxy.get("all_proxy") or ""))

    bot_count_default = len(bots_seed) if bots_seed else 1
    bot_count = _prompt_int("bot count", bot_count_default, min_value=1)
    bots: List[Dict[str, Any]] = []

    existing_bot_ids: set[str] = set()
    for idx in range(bot_count):
        seed_bot = bots_seed[idx] if idx < len(bots_seed) else {}
        bot_payload = _prompt_bot_payload(index=idx + 1, seed_bot=seed_bot, existing_bot_ids=existing_bot_ids)
        existing_bot_ids.add(str(bot_payload.get("bot_id") or ""))
        bots.append(bot_payload)

    print("\n[SETUP] LLM provider config")
    print("[SETUP] You can add more models manually in llm_config later.")
    llm_seed = str(seed.get("llm_config_path") or "")
    llm_default_path = (
        str(Path(runtime_root).expanduser().resolve() / "config" / "llm_config.yaml")
        if _is_placeholder_text(llm_seed)
        else (llm_seed or str(Path(runtime_root).expanduser().resolve() / "config" / "llm_config.yaml"))
    )
    llm_config_path_text = _prompt_text("llm_config_path (will be created/updated)", llm_default_path, required=True)
    llm_config_path = Path(llm_config_path_text).expanduser().resolve()
    llm_base_url = _prompt_text("llm.base_url", "https://openrouter.ai/api/v1", required=True)
    llm_api_key_default = os.environ.get("OPENAI_API_KEY", "")
    llm_api_key = _prompt_text("llm.api_key", llm_api_key_default, required=False)
    llm_model_raw = _prompt_text(
        "llm.model (recommended: full id like 'openai/gpt-4o'; bare names auto-add 'openai/')",
        "openai/google/gemini-3-flash-preview",
        required=True,
    )
    llm_model = _normalize_model_name(llm_model_raw)
    llm_multimodal = _prompt_bool("llm model supports multimodal (image input)", True)
    seed_context = _normalize_context_settings(seed.get("context"))
    print("\n[SETUP] Context compression config")
    context_user_history_tokens = _prompt_int("context.user_history_compress_threshold_tokens", int(seed_context.get("user_history_compress_threshold_tokens", 1500)), min_value=0)
    context_structured_agents = _prompt_int("context.structured_call_info_compress_threshold_agents", int(seed_context.get("structured_call_info_compress_threshold_agents", 10)), min_value=1)
    context_structured_tokens = _prompt_int("context.structured_call_info_compress_threshold_tokens", int(seed_context.get("structured_call_info_compress_threshold_tokens", 2200)), min_value=0)
    if llm_config_path.exists():
        overwrite = _prompt_bool(f"llm config exists, overwrite? {llm_config_path}", True)
        if not overwrite:
            print(f"[SETUP] keep existing llm config: {llm_config_path}")
        else:
            _write_minimal_llm_config(llm_config_path, base_url=llm_base_url, api_key=llm_api_key, model_name=llm_model, multimodal=llm_multimodal)
            print(f"[OK] llm config written: {llm_config_path}")
    else:
        _write_minimal_llm_config(llm_config_path, base_url=llm_base_url, api_key=llm_api_key, model_name=llm_model, multimodal=llm_multimodal)
        print(f"[OK] llm config written: {llm_config_path}")

    manifest: Dict[str, Any] = {
        "llm_config_path": str(llm_config_path),
        "runtime_root": runtime_root,
        "fleet_config_path": fleet_config_path,
        "proxy_env": {"http_proxy": http_proxy, "https_proxy": https_proxy, "all_proxy": all_proxy},
        "context": {
            "user_history_compress_threshold_tokens": int(context_user_history_tokens),
            "structured_call_info_compress_threshold_agents": int(context_structured_agents),
            "structured_call_info_compress_threshold_tokens": int(context_structured_tokens),
        },
        "bots": bots,
    }
    return manifest_path, manifest


def interactive_add_bot(manifest_path: Path) -> str:
    manifest = _load_json(manifest_path)
    bots = manifest.get("bots")
    if not isinstance(bots, list):
        bots = []
        manifest["bots"] = bots

    existing_ids = {
        str(item.get("bot_id") or "").strip()
        for item in bots
        if isinstance(item, dict) and str(item.get("bot_id") or "").strip()
    }
    new_payload = _prompt_bot_payload(index=len(existing_ids) + 1, existing_bot_ids=existing_ids)
    new_bot_id = str(new_payload.get("bot_id") or "").strip()
    bots.append(new_payload)
    _dump_json(manifest_path, manifest)
    return new_bot_id



#!/usr/bin/env python3
from __future__ import annotations

from contextlib import contextmanager
import json
import os
from pathlib import Path
import shlex
import signal
import subprocess
import sys
import time
from typing import Any, Dict, List, Optional, Tuple

try:
    import fcntl  # type: ignore
except Exception:  # pragma: no cover
    fcntl = None  # type: ignore


REPO_ROOT = Path(__file__).resolve().parents[1]
SERVICE_PATH = REPO_ROOT / "cheapclaw_service.py"
FLEET_WEB_CONSOLE_PATH = REPO_ROOT / "scripts" / "fleet_web_console.py"
DEFAULT_HOME_ROOT = (Path.home() / "cheapclaw").resolve()
DEFAULT_RUNTIME_ROOT = (DEFAULT_HOME_ROOT / "runtime").resolve()
PLACEHOLDER_SNIPPETS = (
    "/ABS/PATH/TO/",
    "/ABS/PATH/TO",
    "telegram_token_here",
    "cli_xxx",
)
PROXY_ENV_KEYS = (
    "http_proxy",
    "https_proxy",
    "all_proxy",
    "no_proxy",
    "HTTP_PROXY",
    "HTTPS_PROXY",
    "ALL_PROXY",
    "NO_PROXY",
)


def _is_placeholder_text(text: str) -> bool:
    value = str(text or "").strip()
    if not value:
        return False
    if any(snippet in value for snippet in PLACEHOLDER_SNIPPETS):
        return True
    return value in {"xxx", "your_token_here", "your_app_id", "your_app_secret"}


def _load_json(path: Path) -> Dict[str, Any]:
    try:
        payload = json.loads(path.read_text(encoding="utf-8"))
    except Exception as exc:
        raise SystemExit(f"[ERROR] Failed to parse JSON: {path} ({exc})")
    if not isinstance(payload, dict):
        raise SystemExit(f"[ERROR] JSON root must be an object: {path}")
    return payload


def _dump_json(path: Path, payload: Dict[str, Any]) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(json.dumps(payload, ensure_ascii=False, indent=2) + "\n", encoding="utf-8")


def _required_text(value: Any, field_name: str, bot_id: str) -> str:
    text = str(value or "").strip()
    if not text:
        raise SystemExit(f"[ERROR] Missing '{field_name}' for bot '{bot_id}'")
    return text


def _child_process_env(manifest: Dict[str, Any]) -> Dict[str, str]:
    env = os.environ.copy()
    for key in PROXY_ENV_KEYS:
        env.pop(key, None)

    proxy_env = manifest.get("proxy_env")
    if not isinstance(proxy_env, dict):
        return env

    for key in ("http_proxy", "https_proxy", "all_proxy", "no_proxy"):
        value = str(proxy_env.get(key) or "").strip()
        if not value:
            continue
        env[key] = value
        env[key.upper()] = value
    return env


def _safe_load_json(path: Path, default: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
    fallback = default if isinstance(default, dict) else {}
    if not path.exists():
        return dict(fallback)
    try:
        payload = json.loads(path.read_text(encoding="utf-8"))
    except Exception:
        return dict(fallback)
    if not isinstance(payload, dict):
        return dict(fallback)
    return payload


def safe_load_json(path: Path, default: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
    return _safe_load_json(path, default)


@contextmanager
def file_lock(lock_path: Path):
    lock_path.parent.mkdir(parents=True, exist_ok=True)
    lock_file = open(lock_path, "a+", encoding="utf-8")
    try:
        if fcntl is not None:
            fcntl.flock(lock_file.fileno(), fcntl.LOCK_EX)
        yield
    finally:
        try:
            if fcntl is not None:
                fcntl.flock(lock_file.fileno(), fcntl.LOCK_UN)
        finally:
            lock_file.close()


def bot_process_lock_path(user_data_root: Path) -> Path:
    return user_data_root / "cheapclaw" / "runtime" / "process_ctl.lock"


def fleet_web_runtime_paths(manifest_path: Path) -> Tuple[Path, Path, Path]:
    runtime_root = DEFAULT_RUNTIME_ROOT
    if manifest_path.exists():
        try:
            manifest = _load_json(manifest_path)
            runtime_raw = str(manifest.get("runtime_root") or "").strip()
            if runtime_raw and not _is_placeholder_text(runtime_raw):
                runtime_root = Path(runtime_raw).expanduser().resolve()
        except Exception:
            pass
    state_dir = (runtime_root / ".fleet_web").resolve()
    state_dir.mkdir(parents=True, exist_ok=True)
    return (
        state_dir / "fleet_web.pid",
        state_dir / "fleet_web.log",
        state_dir / "fleet_web.state.json",
    )


def fleet_web_lock_path(manifest_path: Path) -> Path:
    state_dir = fleet_web_runtime_paths(manifest_path)[0].parent
    return state_dir / "process_ctl.lock"


def is_pid_alive(pid: int) -> bool:
    if pid <= 0:
        return False
    try:
        os.kill(pid, 0)
    except OSError:
        return False
    return True


def iter_process_table() -> List[Tuple[int, str]]:
    try:
        proc = subprocess.run(
            ["ps", "-eo", "pid=,command="],
            capture_output=True,
            text=True,
            timeout=3,
            check=False,
        )
    except Exception:
        return []
    if proc.returncode != 0:
        return []
    rows: List[Tuple[int, str]] = []
    for raw in (proc.stdout or "").splitlines():
        line = str(raw or "").strip()
        if not line:
            continue
        parts = line.split(None, 1)
        if not parts:
            continue
        try:
            pid = int(parts[0])
        except Exception:
            continue
        command = parts[1] if len(parts) > 1 else ""
        rows.append((pid, command))
    return rows


def read_pid_command(pid: int) -> str:
    if pid <= 0:
        return ""
    try:
        proc = subprocess.run(
            ["ps", "-p", str(pid), "-o", "command="],
            capture_output=True,
            text=True,
            timeout=2,
            check=False,
        )
    except Exception:
        return ""
    if proc.returncode != 0:
        return ""
    return str(proc.stdout or "").strip()


def read_pid_arg_value(pid: int, arg_name: str) -> str:
    command = read_pid_command(pid)
    if not command:
        return ""
    try:
        tokens = shlex.split(command)
    except Exception:
        return ""
    for idx, token in enumerate(tokens):
        if token == arg_name and idx + 1 < len(tokens):
            return str(tokens[idx + 1]).strip()
        if token.startswith(f"{arg_name}="):
            return str(token.split("=", 1)[1]).strip()
    return ""


def all_needles_in_command(command: str, needles: List[str]) -> bool:
    if not command:
        return False
    return all(str(needle) in command for needle in needles)


def discover_pids_by_needles(needles: List[str]) -> List[int]:
    pids: List[int] = []
    for pid, command in iter_process_table():
        if all_needles_in_command(command, needles):
            pids.append(int(pid))
    return sorted(set(pids))


def bot_service_needles(bot_id: str, user_data_root: Path) -> List[str]:
    return [
        "cheapclaw_service.py",
        "--run-loop",
        f"--bot-id {str(bot_id)}",
        f"--user-data-root {str(user_data_root)}",
    ]


def is_pid_for_bot_service(pid: int, *, bot_id: str, user_data_root: Path) -> bool:
    if not is_pid_alive(pid):
        return False
    command = read_pid_command(pid)
    return all_needles_in_command(command, bot_service_needles(bot_id, user_data_root))


def discover_bot_service_pids(bot_id: str, user_data_root: Path) -> List[int]:
    return discover_pids_by_needles(bot_service_needles(bot_id, user_data_root))


def write_runtime_state_pid(user_data_root: Path, *, pid: int) -> None:
    state_path = user_data_root / "cheapclaw" / "runtime" / "state.json"
    state = _safe_load_json(state_path, {})
    bot_state = state.get("bot") if isinstance(state.get("bot"), dict) else {}
    bot_state["pid"] = int(pid)
    state["bot"] = bot_state
    _dump_json(state_path, state)


def read_bot_pid_from_runtime_state(user_data_root: Path) -> int:
    state_path = user_data_root / "cheapclaw" / "runtime" / "state.json"
    if not state_path.exists():
        return 0
    try:
        state_payload = _load_json(state_path)
    except Exception:
        return 0
    if not isinstance(state_payload, dict):
        return 0
    return int((((state_payload.get("bot") or {}) if isinstance(state_payload, dict) else {}).get("pid")) or 0)


def collect_bot_service_target_pids(bot_id: str, user_data_root: Path) -> List[int]:
    pids = discover_bot_service_pids(bot_id, user_data_root)
    state_pid = read_bot_pid_from_runtime_state(user_data_root)
    if is_pid_for_bot_service(state_pid, bot_id=bot_id, user_data_root=user_data_root):
        pids.append(int(state_pid))
    return sorted(set(pids))


def bot_start_conflicts_with_other_fleet(
    *,
    bot_id: str,
    user_data_root: Path,
    expected_fleet_config_path: Path,
) -> List[Dict[str, Any]]:
    expected = str(expected_fleet_config_path.expanduser().resolve())
    conflicts: List[Dict[str, Any]] = []
    for pid in discover_bot_service_pids(bot_id, user_data_root):
        detected_raw = read_pid_arg_value(pid, "--fleet-config-path")
        if not detected_raw:
            continue
        try:
            detected = str(Path(detected_raw).expanduser().resolve())
        except Exception:
            detected = str(detected_raw).strip()
        if detected and detected != expected:
            conflicts.append(
                {
                    "pid": int(pid),
                    "detected_fleet_config_path": detected,
                    "expected_fleet_config_path": expected,
                }
            )
    return conflicts


def fleet_web_needles(manifest_path: Path) -> List[str]:
    return [
        "fleet_web_console.py",
        f"--manifest {str(manifest_path)}",
    ]


def is_pid_for_fleet_web(pid: int, *, manifest_path: Path) -> bool:
    if not is_pid_alive(pid):
        return False
    command = read_pid_command(pid)
    if "fleet_web_console.py" not in command:
        return False
    detected_raw = read_pid_arg_value(pid, "--manifest")
    if not detected_raw:
        return True
    try:
        detected = str(Path(detected_raw).expanduser().resolve())
    except Exception:
        detected = str(detected_raw).strip()
    expected = str(manifest_path.expanduser().resolve())
    return bool(detected) and detected == expected


def discover_repo_bot_loop_pids() -> List[int]:
    return discover_pids_by_needles([str(SERVICE_PATH), "cheapclaw_service.py", "--run-loop"])


def discover_repo_fleet_web_pids() -> List[int]:
    return discover_pids_by_needles([str(FLEET_WEB_CONSOLE_PATH), "fleet_web_console.py"])


def collect_fleet_web_target_pids(manifest_path: Path) -> List[int]:
    pids = [int(pid) for pid in discover_repo_fleet_web_pids() if is_pid_for_fleet_web(int(pid), manifest_path=manifest_path)]
    pid_path, _log_path, _state_path = fleet_web_runtime_paths(manifest_path)
    pid = 0
    if pid_path.exists():
        try:
            pid = int((pid_path.read_text(encoding="utf-8") or "0").strip())
        except Exception:
            pid = 0
    if is_pid_for_fleet_web(pid, manifest_path=manifest_path):
        pids.append(int(pid))
    return sorted(set(pids))


def fleet_web_status_payload(manifest_path: Path) -> Dict[str, Any]:
    pid_path, log_path, state_path = fleet_web_runtime_paths(manifest_path)
    pid = 0
    if pid_path.exists():
        try:
            pid = int((pid_path.read_text(encoding="utf-8") or "0").strip())
        except Exception:
            pid = 0
    target_pids = collect_fleet_web_target_pids(manifest_path)
    if target_pids:
        pid = int(target_pids[0])
    else:
        pid = 0
    running = bool(target_pids)
    state = _safe_load_json(state_path, {})
    return {
        "status": "success",
        "manifest_path": str(manifest_path),
        "running": bool(running),
        "pid": int(pid),
        "pids": target_pids,
        "host": str(state.get("host") or ""),
        "port": int(state.get("port") or 0) if state.get("port") else 0,
        "url": str(state.get("url") or ""),
        "log_path": str(log_path),
        "state_path": str(state_path),
    }


def start_fleet_web(manifest_path: Path, *, host: str, port: int) -> Dict[str, Any]:
    with file_lock(fleet_web_lock_path(manifest_path)):
        pid_path, log_path, state_path = fleet_web_runtime_paths(manifest_path)
        existing = fleet_web_status_payload(manifest_path)
        if bool(existing.get("running")):
            return {**existing, "status": "already_running", "output": f"fleet web already running at {existing.get('url') or '-'}"}
        if not FLEET_WEB_CONSOLE_PATH.exists():
            return {"status": "error", "manifest_path": str(manifest_path), "error": f"fleet web script not found: {FLEET_WEB_CONSOLE_PATH}"}

        cmd = [
            sys.executable,
            str(FLEET_WEB_CONSOLE_PATH),
            "--manifest",
            str(manifest_path),
            "--host",
            str(host or "127.0.0.1"),
            "--port",
            str(int(port or 8787)),
        ]
        env = os.environ.copy()
        log_path.parent.mkdir(parents=True, exist_ok=True)
        std_log = open(log_path, "a", encoding="utf-8")
        process = subprocess.Popen(cmd, cwd=str(REPO_ROOT), env=env, stdout=std_log, stderr=std_log, start_new_session=True)
        std_log.close()
        pid_path.write_text(str(int(process.pid)), encoding="utf-8")
        payload = {
            "host": str(host or "127.0.0.1"),
            "port": int(port or 8787),
            "url": f"http://{str(host or '127.0.0.1')}:{int(port or 8787)}/dashboard",
            "manifest_path": str(manifest_path),
            "pid": int(process.pid),
            "started_at": int(time.time()),
        }
        _dump_json(state_path, payload)
        time.sleep(0.4)
        if process.poll() is not None:
            try:
                log_tail = "\n".join(log_path.read_text(encoding="utf-8", errors="ignore").splitlines()[-20:])
            except Exception:
                log_tail = ""
            return {
                "status": "error",
                "manifest_path": str(manifest_path),
                "running": False,
                "pid": int(process.pid),
                "host": payload["host"],
                "port": payload["port"],
                "url": payload["url"],
                "log_path": str(log_path),
                "error": "fleet web console exited immediately after start",
                "log_tail": log_tail,
            }
        return {
            "status": "started",
            "manifest_path": str(manifest_path),
            "running": True,
            "pid": int(process.pid),
            "host": payload["host"],
            "port": payload["port"],
            "url": payload["url"],
            "log_path": str(log_path),
        }


def stop_fleet_web(manifest_path: Path, *, timeout_sec: int = 8) -> Dict[str, Any]:
    with file_lock(fleet_web_lock_path(manifest_path)):
        pid_path, log_path, state_path = fleet_web_runtime_paths(manifest_path)
        pids = collect_fleet_web_target_pids(manifest_path)
        if not pids:
            status = fleet_web_status_payload(manifest_path)
            return {**status, "status": "already_stopped", "log_path": str(log_path)}
        for pid in pids:
            try:
                os.kill(pid, signal.SIGTERM)
            except ProcessLookupError:
                continue

        deadline = time.time() + max(1, int(timeout_sec))
        remaining: List[int] = list(pids)
        while time.time() < deadline:
            remaining = [pid for pid in pids if is_pid_for_fleet_web(pid, manifest_path=manifest_path)]
            if not remaining:
                break
            time.sleep(0.2)
        if remaining:
            try:
                for pid in remaining:
                    try:
                        os.kill(pid, signal.SIGKILL)
                    except ProcessLookupError:
                        continue
            except Exception:
                pass
        time.sleep(0.2)
        final_remaining = [pid for pid in pids if is_pid_for_fleet_web(pid, manifest_path=manifest_path)]
        try:
            pid_path.unlink(missing_ok=True)
        except Exception:
            pass
        if state_path.exists():
            state = _safe_load_json(state_path, {})
            state["stopped_at"] = int(time.time())
            _dump_json(state_path, state)
        return {
            "status": "stopped" if not final_remaining else "partial",
            "manifest_path": str(manifest_path),
            "pid": int(pids[0]),
            "pids": pids,
            "remaining_pids": final_remaining,
            "log_path": str(log_path),
        }


def resolve_bot_from_manifest(manifest_path: Path, bot_id: str) -> Tuple[Dict[str, Any], Dict[str, Any], Path, Path]:
    manifest = _load_json(manifest_path)
    llm_config_path = Path(_required_text(manifest.get("llm_config_path"), "llm_config_path", "<manifest>")).expanduser().resolve()
    runtime_root = Path(_required_text(manifest.get("runtime_root"), "runtime_root", "<manifest>")).expanduser().resolve()
    fleet_config_path = Path(str(manifest.get("fleet_config_path") or (runtime_root / "fleet.generated.json"))).expanduser().resolve()
    target: Dict[str, Any] | None = None
    for item in manifest.get("bots", []):
        if isinstance(item, dict) and str(item.get("bot_id") or "").strip() == bot_id:
            target = item
            break
    if target is None:
        raise SystemExit(f"[ERROR] bot_id not found in manifest: {bot_id}")
    user_data_root = Path(runtime_root) / str(target.get("bot_id") or bot_id)
    return manifest, target, user_data_root.expanduser().resolve(), llm_config_path


def list_manifest_bots(manifest_path: Path, *, enabled_only: bool = False) -> List[Dict[str, Any]]:
    manifest = _load_json(manifest_path)
    bots = [item for item in manifest.get("bots", []) if isinstance(item, dict)]
    if enabled_only:
        bots = [item for item in bots if bool(item.get("enabled", True))]
    return bots


def read_bot_runtime_snapshot(user_data_root: Path, *, bot_id: str) -> Dict[str, Any]:
    runtime_root = user_data_root / "cheapclaw" / "runtime"
    state_payload = _safe_load_json(runtime_root / "state.json")
    panel_payload = _safe_load_json(user_data_root / "cheapclaw" / "panel" / "panel.json")
    instructions_payload = _safe_load_json(user_data_root / "cheapclaw" / "monitor_instructions.json", {"version": 1, "instructions": []})
    instructions = instructions_payload.get("instructions")
    if not isinstance(instructions, list):
        instructions = []
    pending_count = sum(1 for item in instructions if isinstance(item, dict) and str(item.get("status") or "") == "pending")
    bot_runtime = state_payload.get("bot") if isinstance(state_payload.get("bot"), dict) else {}
    service_state = panel_payload.get("service_state") if isinstance(panel_payload.get("service_state"), dict) else {}
    state_pid = int(bot_runtime.get("pid") or 0)
    live_pids = discover_bot_service_pids(bot_id, user_data_root)
    pid = int(live_pids[0]) if live_pids else 0
    if pid <= 0 and is_pid_for_bot_service(state_pid, bot_id=bot_id, user_data_root=user_data_root):
        pid = int(state_pid)
    return {
        "pid": pid,
        "alive": bool(pid > 0),
        "pids": live_pids,
        "pending_instruction_count": pending_count,
        "main_agent_running": bool(service_state.get("main_agent_running")),
        "main_agent_last_started_at": str(service_state.get("main_agent_last_started_at") or ""),
        "watchdog_last_run_at": str(service_state.get("watchdog_last_run_at") or ""),
        "service_log_path": str(runtime_root / "cheapclaw_service.log"),
        "loop_log_path": str(runtime_root / "service.loop.log"),
    }


def status_single_bot_from_manifest(manifest_path: Path, bot_id: str) -> Dict[str, Any]:
    _manifest, target, user_data_root, _llm = resolve_bot_from_manifest(manifest_path, bot_id)
    runtime = read_bot_runtime_snapshot(user_data_root, bot_id=bot_id)
    return {
        "bot_id": str(target.get("bot_id") or bot_id),
        "display_name": str(target.get("display_name") or bot_id),
        "channel": str(target.get("channel") or ""),
        "enabled": bool(target.get("enabled", True)),
        "serve_webhooks": bool(target.get("serve_webhooks", False)),
        "user_data_root": str(user_data_root),
        **runtime,
    }


def status_all_bots_from_manifest(manifest_path: Path) -> Dict[str, Any]:
    bots = list_manifest_bots(manifest_path, enabled_only=False)
    statuses = []
    for item in bots:
        bot_id = str(item.get("bot_id") or "").strip()
        if not bot_id:
            continue
        statuses.append(status_single_bot_from_manifest(manifest_path, bot_id))
    return {
        "status": "success",
        "manifest_path": str(manifest_path),
        "bot_count": len(statuses),
        "running_count": sum(1 for item in statuses if item.get("alive")),
        "bots": statuses,
    }


def stop_single_bot_from_manifest(manifest_path: Path, *, bot_id: str, timeout_sec: int = 8) -> Dict[str, Any]:
    _manifest, _target, user_data_root, _llm = resolve_bot_from_manifest(manifest_path, bot_id)
    with file_lock(bot_process_lock_path(user_data_root)):
        pids = collect_bot_service_target_pids(bot_id, user_data_root)
        if not pids:
            stale_pid = read_bot_pid_from_runtime_state(user_data_root)
            if stale_pid:
                try:
                    write_runtime_state_pid(user_data_root, pid=0)
                except Exception:
                    pass
            return {"status": "already_stopped", "bot_id": bot_id, "pid": stale_pid, "pids": [], "user_data_root": str(user_data_root)}

        for pid in pids:
            try:
                os.kill(pid, signal.SIGTERM)
            except ProcessLookupError:
                continue

        deadline = time.time() + max(1, int(timeout_sec))
        remaining: List[int] = list(pids)
        while time.time() < deadline:
            remaining = [pid for pid in pids if is_pid_for_bot_service(pid, bot_id=bot_id, user_data_root=user_data_root)]
            if not remaining:
                try:
                    write_runtime_state_pid(user_data_root, pid=0)
                except Exception:
                    pass
                return {"status": "stopped", "bot_id": bot_id, "pid": int(pids[0]), "pids": pids, "user_data_root": str(user_data_root)}
            time.sleep(0.2)

        for pid in remaining:
            try:
                os.kill(pid, signal.SIGKILL)
            except ProcessLookupError:
                continue
        time.sleep(0.2)
        final_remaining = [pid for pid in pids if is_pid_for_bot_service(pid, bot_id=bot_id, user_data_root=user_data_root)]
        if not final_remaining:
            try:
                write_runtime_state_pid(user_data_root, pid=0)
            except Exception:
                pass
        return {
            "status": "killed" if not final_remaining else "partial",
            "bot_id": bot_id,
            "pid": int(pids[0]),
            "pids": pids,
            "remaining_pids": final_remaining,
            "user_data_root": str(user_data_root),
        }


def start_single_bot_from_manifest(manifest_path: Path, *, bot_id: str, poll_interval: int = 5) -> Dict[str, Any]:
    manifest, target, user_data_root, llm_config_path = resolve_bot_from_manifest(manifest_path, bot_id)
    runtime_root = Path(_required_text(manifest.get("runtime_root"), "runtime_root", "<manifest>")).expanduser().resolve()
    fleet_config_path = Path(str(manifest.get("fleet_config_path") or (runtime_root / "fleet.generated.json"))).expanduser().resolve()
    if not bool(target.get("enabled", True)):
        raise SystemExit(f"[ERROR] bot is disabled in manifest: {bot_id}")

    with file_lock(bot_process_lock_path(user_data_root)):
        conflicts = bot_start_conflicts_with_other_fleet(bot_id=bot_id, user_data_root=user_data_root, expected_fleet_config_path=fleet_config_path)
        if conflicts:
            conflict_pids = sorted(set(int(item.get("pid") or 0) for item in conflicts if int(item.get("pid") or 0) > 0))
            return {
                "status": "conflict",
                "bot_id": bot_id,
                "pid": int(conflict_pids[0]) if conflict_pids else 0,
                "pids": conflict_pids,
                "user_data_root": str(user_data_root),
                "fleet_config_path": str(fleet_config_path),
                "conflicts": conflicts,
                "error": "bot process is already running under a different fleet config; stop the conflicting process first",
            }
        current_pid = read_bot_pid_from_runtime_state(user_data_root)
        if is_pid_for_bot_service(current_pid, bot_id=bot_id, user_data_root=user_data_root):
            return {"status": "already_running", "bot_id": bot_id, "pid": current_pid, "user_data_root": str(user_data_root)}
        live_pids = discover_bot_service_pids(bot_id, user_data_root)
        if live_pids:
            try:
                write_runtime_state_pid(user_data_root, pid=int(live_pids[0]))
            except Exception:
                pass
            return {"status": "already_running", "bot_id": bot_id, "pid": int(live_pids[0]), "pids": live_pids, "user_data_root": str(user_data_root)}

        cmd = [
            sys.executable,
            str(SERVICE_PATH),
            "--user-data-root",
            str(user_data_root),
            "--llm-config-path",
            str(llm_config_path),
            "--bot-id",
            str(bot_id),
            "--bot-display-name",
            str(target.get("display_name") or bot_id),
            "--fleet-config-path",
            str(fleet_config_path),
            "--run-loop",
            "--poll-interval",
            str(max(1, int(poll_interval))),
        ]
        if bool(target.get("serve_webhooks", False)):
            cmd.extend(
                [
                    "--serve-webhooks",
                    "--host",
                    str(target.get("host") or "127.0.0.1"),
                    "--port",
                    str(int(target.get("port") or 8765)),
                ]
            )

        env = _child_process_env(manifest)
        log_dir = user_data_root / "cheapclaw" / "runtime"
        log_dir.mkdir(parents=True, exist_ok=True)
        std_log_path = log_dir / "service.loop.log"
        std_log = open(std_log_path, "a", encoding="utf-8")
        process = subprocess.Popen(cmd, cwd=str(REPO_ROOT), env=env, stdout=std_log, stderr=std_log, start_new_session=True)
        std_log.close()
        return {
            "status": "started",
            "bot_id": bot_id,
            "pid": int(process.pid),
            "user_data_root": str(user_data_root),
            "log_path": str(std_log_path),
            "fleet_config_path": str(fleet_config_path),
        }


def start_all_bots_from_manifest(manifest_path: Path, *, poll_interval: int = 5) -> Dict[str, Any]:
    bots = list_manifest_bots(manifest_path, enabled_only=True)
    results = []
    for item in bots:
        bot_id = str(item.get("bot_id") or "").strip()
        if not bot_id:
            continue
        results.append(start_single_bot_from_manifest(manifest_path, bot_id=bot_id, poll_interval=poll_interval))
    return {
        "status": "success",
        "manifest_path": str(manifest_path),
        "requested_bot_count": len(bots),
        "started_count": sum(1 for item in results if str(item.get("status") or "") == "started"),
        "already_running_count": sum(1 for item in results if str(item.get("status") or "") == "already_running"),
        "conflict_count": sum(1 for item in results if str(item.get("status") or "") == "conflict"),
        "results": results,
    }


def stop_all_bots_from_manifest(manifest_path: Path, *, timeout_sec: int = 8) -> Dict[str, Any]:
    bots = list_manifest_bots(manifest_path, enabled_only=False)
    results = []
    for item in bots:
        bot_id = str(item.get("bot_id") or "").strip()
        if not bot_id:
            continue
        results.append(stop_single_bot_from_manifest(manifest_path, bot_id=bot_id, timeout_sec=timeout_sec))
    return {
        "status": "success",
        "manifest_path": str(manifest_path),
        "requested_bot_count": len(bots),
        "stopped_count": sum(1 for item in results if str(item.get("status") or "") in {"stopped", "killed"}),
        "already_stopped_count": sum(1 for item in results if str(item.get("status") or "") == "already_stopped"),
        "partial_count": sum(1 for item in results if str(item.get("status") or "") == "partial"),
        "results": results,
    }


def terminate_pids(pids: List[int], *, timeout_sec: int = 8) -> Dict[str, Any]:
    targets = sorted(set(int(pid) for pid in pids if is_pid_alive(int(pid))))
    if not targets:
        return {"status": "already_stopped", "pids": [], "remaining_pids": []}
    for pid in targets:
        try:
            os.kill(pid, signal.SIGTERM)
        except ProcessLookupError:
            continue
    deadline = time.time() + max(1, int(timeout_sec))
    remaining = list(targets)
    while time.time() < deadline:
        remaining = [pid for pid in targets if is_pid_alive(pid)]
        if not remaining:
            break
        time.sleep(0.2)
    if remaining:
        for pid in remaining:
            try:
                os.kill(pid, signal.SIGKILL)
            except ProcessLookupError:
                continue
        time.sleep(0.2)
    final_remaining = [pid for pid in targets if is_pid_alive(pid)]
    return {"status": "stopped" if not final_remaining else "partial", "pids": targets, "remaining_pids": final_remaining}


def stop_fleet_web_fallback(*, timeout_sec: int = 8) -> Dict[str, Any]:
    payload = terminate_pids(discover_repo_fleet_web_pids(), timeout_sec=timeout_sec)
    return {
        "status": payload.get("status") or "already_stopped",
        "running": bool(payload.get("remaining_pids")),
        "mode": "manifest_missing_fallback",
        "manifest_path": "",
        "pid": int((payload.get("pids") or [0])[0]) if payload.get("pids") else 0,
        "pids": payload.get("pids") or [],
        "remaining_pids": payload.get("remaining_pids") or [],
        "log_path": "",
        "note": "manifest not found, stopped web by process scan",
    }


def stop_all_fallback_without_manifest(*, timeout_sec: int = 8) -> Dict[str, Any]:
    bot_payload = terminate_pids(discover_repo_bot_loop_pids(), timeout_sec=timeout_sec)
    web_payload = terminate_pids(discover_repo_fleet_web_pids(), timeout_sec=timeout_sec)
    return {
        "status": "success",
        "mode": "manifest_missing_fallback",
        "manifest_path": "",
        "bot_processes": bot_payload,
        "web_processes": web_payload,
        "note": "manifest not found, fallback stop by process scan",
    }



#!/usr/bin/env python3
from pathlib import Path
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import generate_task_id
from tool_server_lite.tools.file_tools import BaseTool

class CheapClawGenerateTaskIdTool(BaseTool):
    name = "cheapclaw_generate_task_id"
    def execute(self, task_id, parameters):
        channel = str(parameters.get("channel") or "").strip()
        conversation_id = str(parameters.get("conversation_id") or "").strip()
        task_name = str(parameters.get("task_name") or "").strip()
        if not channel or not conversation_id or not task_name:
            return {"status": "error", "output": "", "error": "channel, conversation_id and task_name are required"}
        value = generate_task_id(channel, conversation_id, task_name)
        return {"status": "success", "output": value, "task_id": value}



#!/usr/bin/env python3
from __future__ import annotations

import json
import os
import tempfile
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path
from typing import Any, Callable, Dict, Optional


def now_iso() -> str:
    return datetime.now().astimezone().isoformat(timespec="seconds")


def parse_iso(value: str) -> Optional[datetime]:
    text = str(value or "").strip()
    if not text:
        return None
    try:
        parsed = datetime.fromisoformat(text)
        if parsed.tzinfo is None:
            return parsed.replace(tzinfo=datetime.now().astimezone().tzinfo)
        return parsed
    except Exception:
        return None


def resolve_user_data_root(*, explicit: str | Path | None = None, helper_file: str | Path | None = None) -> Path:
    if explicit:
        return Path(explicit).expanduser().resolve()

    for env_key in ("CHEAPCLAW_USER_DATA_ROOT", "MLA_USER_DATA_ROOT"):
        env_root = os.environ.get(env_key, "").strip()
        if env_root:
            return Path(env_root).expanduser().resolve()

    helper_parent = Path(helper_file).resolve().parent if helper_file else Path(__file__).resolve().parent
    if (helper_parent / "cheapclaw").exists():
        return helper_parent

    return (Path.home() / "mla_v3").resolve()


def slugify(value: str, fallback: str = "item", max_len: int = 80) -> str:
    text = str(value or "").strip()
    if not text:
        return fallback
    chars = []
    last_dash = False
    for ch in text:
        if ch.isalnum():
            chars.append(ch.lower())
            last_dash = False
        elif ch in {"-", "_"}:
            chars.append(ch)
            last_dash = False
        else:
            if not last_dash:
                chars.append("-")
            last_dash = True
    return "".join(chars).strip("-_")[:max_len] or fallback


def is_relative_to(path: Path, parent: Path) -> bool:
    try:
        path.relative_to(parent)
        return True
    except ValueError:
        return False


def atomic_write_text(path: Path, content: str) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    fd, tmp_name = tempfile.mkstemp(prefix=path.name + ".", dir=str(path.parent))
    try:
        with os.fdopen(fd, "w", encoding="utf-8") as tmp_file:
            tmp_file.write(content)
        os.replace(tmp_name, path)
    finally:
        if os.path.exists(tmp_name):
            os.unlink(tmp_name)


def atomic_write_json(path: Path, payload: Dict[str, Any]) -> None:
    atomic_write_text(path, json.dumps(payload, ensure_ascii=False, indent=2))


def load_json(path: Path, default: Any) -> Any:
    try:
        return json.loads(path.read_text(encoding="utf-8"))
    except Exception:
        return default


@dataclass(frozen=True)
class CheapClawPaths:
    user_data_root: Path
    cheapclaw_root: Path
    tools_root: Path
    panel_dir: Path
    panel_path: Path
    panel_lock_path: Path
    panel_backups_dir: Path
    plans_path: Path
    config_dir: Path
    app_config_path: Path
    app_config_example_path: Path
    channels_config_path: Path
    channels_example_path: Path
    channels_root: Path
    outbox_dir: Path
    tasks_root: Path
    task_skills_root: Path
    supervisor_task_id: Path
    monitor_instructions_path: Path
    monitor_system_add_path: Path
    runtime_dir: Path
    runtime_state_path: Path

    @classmethod
    def from_user_data_root(cls, user_data_root: str | Path, app_name: str = "cheapclaw") -> "CheapClawPaths":
        root = Path(user_data_root).expanduser().resolve()
        cheapclaw_root = root / app_name
        panel_dir = cheapclaw_root / "panel"
        runtime_dir = cheapclaw_root / "runtime"
        config_dir = cheapclaw_root / "config"
        return cls(
            user_data_root=root,
            cheapclaw_root=cheapclaw_root,
            tools_root=root / "tools_library",
            panel_dir=panel_dir,
            panel_path=panel_dir / "panel.json",
            panel_lock_path=panel_dir / "panel.lock",
            panel_backups_dir=panel_dir / "backups",
            plans_path=cheapclaw_root / "plans.json",
            config_dir=config_dir,
            app_config_path=config_dir / "app_config.json",
            app_config_example_path=config_dir / "app_config.example.json",
            channels_config_path=config_dir / "channels.json",
            channels_example_path=config_dir / "channels.example.json",
            channels_root=cheapclaw_root / "channels",
            outbox_dir=cheapclaw_root / "outbox",
            tasks_root=cheapclaw_root / "tasks",
            task_skills_root=cheapclaw_root / "task_skills",
            supervisor_task_id=cheapclaw_root / "supervisor_task",
            monitor_instructions_path=cheapclaw_root / "monitor_instructions.json",
            monitor_system_add_path=cheapclaw_root / "supervisor_task" / "system-add.md",
            runtime_dir=runtime_dir,
            runtime_state_path=runtime_dir / "state.json",
        )


def _default_panel(paths: CheapClawPaths) -> Dict[str, Any]:
    return {
        "version": 1,
        "channels": {},
        "service_state": {
            "main_agent_task_id": str(paths.supervisor_task_id),
            "main_agent_running": False,
            "main_agent_run_id": "",
            "main_agent_last_started_at": "",
            "main_agent_last_finished_at": "",
            "watchdog_last_run_at": "",
            "last_backup_path": "",
        },
    }


def _normalize_conversation(paths: CheapClawPaths, channel: str, conversation_id: str, conversation: Dict[str, Any]) -> Dict[str, Any]:
    if not isinstance(conversation, dict):
        conversation = {}
    defaults = {
        "channel": channel,
        "conversation_id": conversation_id,
        "conversation_type": "group",
        "display_name": conversation_id,
        "trigger_policy": {"require_mention": True},
        "message_history_path": str(paths.channels_root / slugify(channel) / slugify(conversation_id) / "social_history.jsonl"),
        "context_summary_path": str(paths.channels_root / slugify(channel) / slugify(conversation_id) / "latest_context.md"),
        "messages": [],
        "linked_tasks": [],
        "pending_events": [],
        "last_snapshot_path": "",
        "updated_at": "",
        "running_task_count": 0,
        "has_stale_running_tasks": False,
        "latest_user_message_at": "",
        "latest_bot_message_at": "",
        "unread_event_count": 0,
        "last_reply_summary": "",
        "conversation_tags": [],
        "message_task_bindings": [],
    }
    for key, value in defaults.items():
        conversation.setdefault(key, value)
    conversation.pop("dirty", None)
    normalized_tasks = []
    for item in conversation.get("linked_tasks", []):
        if not isinstance(item, dict):
            continue
        task_id = str(item.get("task_id") or "").strip()
        if not task_id:
            continue
        task_defaults = {
            "task_id": task_id,
            "created_at": "",
            "agent_system": "",
            "agent_name": "",
            "status": "unknown",
            "share_context_path": "",
            "stack_path": "",
            "log_path": "",
            "skills_dir": "",
            "default_exposed_skills": [],
            "mcp_servers": [],
            "last_thinking": "",
            "last_thinking_at": "",
            "last_final_output": "",
            "last_final_output_at": "",
            "last_action_at": "",
            "last_log_at": "",
            "fresh_retry_count": 0,
            "last_watchdog_note": "",
            "pid_alive": None,
            "watchdog_observation": "",
            "watchdog_suspected_state": "",
        }
        task_defaults.update(item)
        normalized_tasks.append(task_defaults)
    conversation["linked_tasks"] = normalized_tasks
    conversation["running_task_count"] = sum(1 for item in normalized_tasks if item.get("status") == "running")
    conversation["unread_event_count"] = len(conversation.get("pending_events", []))
    return conversation


def normalize_panel(paths: CheapClawPaths, panel: Dict[str, Any]) -> Dict[str, Any]:
    if not isinstance(panel, dict):
        panel = {}
    panel.setdefault("version", 1)
    panel.setdefault("channels", {})
    panel.setdefault("service_state", {})
    defaults = {
        "main_agent_task_id": str(paths.supervisor_task_id),
        "main_agent_running": False,
        "main_agent_run_id": "",
        "main_agent_last_started_at": "",
        "main_agent_last_finished_at": "",
        "watchdog_last_run_at": "",
        "last_backup_path": "",
    }
    for key, value in defaults.items():
        panel["service_state"].setdefault(key, value)
    panel["service_state"].pop("main_agent_dirty", None)
    for channel, payload in list(panel["channels"].items()):
        if not isinstance(payload, dict):
            panel["channels"][channel] = {"conversations": {}}
            payload = panel["channels"][channel]
        payload.setdefault("conversations", {})
        for conversation_id, conversation in list(payload["conversations"].items()):
            payload["conversations"][conversation_id] = _normalize_conversation(paths, channel, conversation_id, conversation)
    return panel


def normalize_monitor_instructions(payload: Dict[str, Any]) -> Dict[str, Any]:
    if not isinstance(payload, dict):
        payload = {"version": 1, "instructions": []}
    payload.setdefault("version", 1)
    items = payload.get("instructions")
    if not isinstance(items, list):
        items = []
    payload["instructions"] = [item for item in items if isinstance(item, dict)]
    return payload


def ensure_runtime_layout(paths: CheapClawPaths) -> None:
    for path in [
        paths.cheapclaw_root,
        paths.tools_root,
        paths.panel_dir,
        paths.panel_backups_dir,
        paths.config_dir,
        paths.runtime_dir,
        paths.channels_root,
        paths.outbox_dir,
        paths.tasks_root,
        paths.task_skills_root,
        paths.supervisor_task_id,
    ]:
        path.mkdir(parents=True, exist_ok=True)
    if not paths.panel_path.exists():
        atomic_write_json(paths.panel_path, _default_panel(paths))
    if not paths.plans_path.exists():
        atomic_write_json(paths.plans_path, {"version": 1, "plans": []})
    if not paths.runtime_state_path.exists():
        atomic_write_json(paths.runtime_state_path, {"webhook_server": {}, "telegram_offsets": {}})
    if not paths.monitor_instructions_path.exists():
        atomic_write_json(paths.monitor_instructions_path, {"version": 1, "instructions": []})


def _write_panel_backup(paths: CheapClawPaths) -> Path:
    backup_path = paths.panel_backups_dir / f"panel_{datetime.now().strftime('%Y%m%d_%H%M%S_%f')}.json"
    backup_path.write_text(paths.panel_path.read_text(encoding="utf-8"), encoding="utf-8")
    return backup_path


def load_panel_for_paths(paths: CheapClawPaths) -> Dict[str, Any]:
    ensure_runtime_layout(paths)
    panel = load_json(paths.panel_path, _default_panel(paths))
    return normalize_panel(paths, panel)


def save_panel_for_paths(paths: CheapClawPaths, panel: Dict[str, Any], *, backup: bool = True) -> Dict[str, Any]:
    ensure_runtime_layout(paths)
    normalized = normalize_panel(paths, panel)
    if backup and paths.panel_path.exists():
        backup_path = _write_panel_backup(paths)
        normalized.setdefault("service_state", {})["last_backup_path"] = str(backup_path)
    atomic_write_json(paths.panel_path, normalized)
    return normalized


def mutate_panel_for_paths(
    paths: CheapClawPaths,
    updater: Callable[[Dict[str, Any]], Dict[str, Any] | None],
    *,
    backup: bool = True,
) -> Dict[str, Any]:
    current = load_panel_for_paths(paths)
    updated = updater(current)
    panel = current if updated is None else updated
    return save_panel_for_paths(paths, panel, backup=backup)


def load_monitor_instructions_from_path(path: Path) -> Dict[str, Any]:
    if not path.exists():
        return {"version": 1, "instructions": []}
    payload = load_json(path, {"version": 1, "instructions": []})
    return normalize_monitor_instructions(payload)


def save_monitor_instructions_to_path(path: Path, payload: Dict[str, Any]) -> Dict[str, Any]:
    normalized = normalize_monitor_instructions(payload)
    atomic_write_json(path, normalized)
    return normalized


def load_monitor_instructions_for_paths(paths: CheapClawPaths) -> Dict[str, Any]:
    ensure_runtime_layout(paths)
    return load_monitor_instructions_from_path(paths.monitor_instructions_path)


def save_monitor_instructions_for_paths(paths: CheapClawPaths, payload: Dict[str, Any]) -> Dict[str, Any]:
    ensure_runtime_layout(paths)
    return save_monitor_instructions_to_path(paths.monitor_instructions_path, payload)



#!/usr/bin/env python3
from __future__ import annotations

import requests

from _common import API_ROOT, api_request, build_auth_parser, dump, tenant_access_token


def main() -> int:
    parser = build_auth_parser("List Feishu calendar events")
    parser.add_argument("--calendar-id", required=True)
    parser.add_argument("--page-size", type=int, default=50)
    parser.add_argument("--page-token", default="")
    parser.add_argument("--start", default="")
    parser.add_argument("--end", default="")
    args = parser.parse_args()
    token = tenant_access_token(args.app_id, args.app_secret)
    params = {"page_size": max(1, int(args.page_size or 50))}
    if args.page_token:
        params["page_token"] = args.page_token
    if args.start:
        params["start_time"] = args.start
    if args.end:
        params["end_time"] = args.end
    payload = api_request(
        "GET",
        f"{API_ROOT}/calendar/v4/calendars/{args.calendar_id}/events",
        token=token,
        params=params,
    )
    dump(payload)
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
from pathlib import Path
from infiagent import infiagent
import sys
from datetime import datetime

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import assert_managed_task_id, load_panel
from tool_server_lite.tools.file_tools import BaseTool


def _iso_ts(value):
    text = str(value or "").strip()
    if not text:
        return 0.0
    try:
        return datetime.fromisoformat(text).timestamp()
    except Exception:
        return 0.0


def _latest_root_final_output(share_context_path):
    path = Path(str(share_context_path or "")).expanduser()
    if not path.exists():
        return {"resolved": False}
    try:
        payload = __import__("json").loads(path.read_text(encoding="utf-8"))
    except Exception:
        return {"resolved": False}
    if not isinstance(payload, dict):
        return {"resolved": False}

    records = []

    def _collect(entry, source, sequence):
        agents_status = entry.get("agents_status", {})
        if not isinstance(agents_status, dict):
            return
        roots = []
        for agent_id, agent in agents_status.items():
            if not isinstance(agent, dict):
                continue
            if str(agent.get("parent_id") or "").strip():
                continue
            roots.append((str(agent_id), agent))
        if not roots:
            return
        roots.sort(key=lambda item: (_iso_ts(item[1].get("start_time")), _iso_ts(item[1].get("end_time"))))
        root_agent_id, root_agent = roots[-1]
        run_start_at = str(entry.get("start_time") or root_agent.get("start_time") or "")
        run_completed_at = str(entry.get("completion_time") or root_agent.get("end_time") or "")
        final_output = str(root_agent.get("final_output") or "")
        final_output_at = str(root_agent.get("end_time") or run_completed_at or "")
        records.append(
            {
                "source": source,
                "sequence": sequence,
                "run_start_ts": _iso_ts(run_start_at),
                "final_output_ts": _iso_ts(final_output_at),
                "final_output": final_output,
                "final_output_at": final_output_at,
                "root_agent_id": root_agent_id,
                "root_agent_name": str(root_agent.get("agent_name") or ""),
            }
        )

    history = payload.get("history", [])
    if isinstance(history, list):
        for idx, entry in enumerate(history):
            if isinstance(entry, dict):
                _collect(entry, "history", idx)
    current = payload.get("current", {})
    if isinstance(current, dict):
        _collect(current, "current", len(records))

    if not records:
        return {"resolved": True, "final_output": "", "final_output_at": "", "root_agent_id": "", "root_agent_name": "", "source": ""}
    records_with_output = [item for item in records if str(item.get("final_output") or "").strip()]
    if not records_with_output:
        return {"resolved": True, "final_output": "", "final_output_at": "", "root_agent_id": "", "root_agent_name": "", "source": ""}
    latest = max(
        records_with_output,
        key=lambda item: (float(item.get("run_start_ts") or 0.0), float(item.get("final_output_ts") or 0.0), int(item.get("sequence") or 0)),
    )
    return {
        "resolved": True,
        "final_output": str(latest.get("final_output") or ""),
        "final_output_at": str(latest.get("final_output_at") or ""),
        "root_agent_id": str(latest.get("root_agent_id") or ""),
        "root_agent_name": str(latest.get("root_agent_name") or ""),
        "source": str(latest.get("source") or ""),
    }


class CheapClawGetTaskStatusTool(BaseTool):
    name = "cheapclaw_get_task_status"
    def execute(self, task_id, parameters):
        try:
            target_task_id = assert_managed_task_id(str(Path(parameters.get("task_id") or "").expanduser().resolve()), allow_monitor_task=True)
        except ValueError as exc:
            return {"status": "error", "output": "", "error": str(exc)}
        if not target_task_id:
            return {"status": "error", "output": "", "error": "task_id is required"}
        agent = infiagent(user_data_root=str(Path(__import__('os').environ.get('MLA_USER_DATA_ROOT','~/mla_v3')).expanduser().resolve()))
        snapshot = agent.task_snapshot(task_id=target_task_id)
        root_final = _latest_root_final_output(snapshot.get("share_context_path", ""))
        if root_final.get("resolved"):
            snapshot["last_final_output"] = str(root_final.get("final_output") or "")
            snapshot["last_final_output_at"] = str(root_final.get("final_output_at") or "")
            snapshot["last_final_output_root_agent_id"] = str(root_final.get("root_agent_id") or "")
            snapshot["last_final_output_root_agent_name"] = str(root_final.get("root_agent_name") or "")
            snapshot["last_final_output_source"] = str(root_final.get("source") or "")
        log_path = ""
        panel = load_panel()
        for channel_payload in panel.get("channels", {}).values():
            for conv in channel_payload.get("conversations", {}).values():
                for item in conv.get("linked_tasks", []):
                    if item.get("task_id") == target_task_id:
                        log_path = item.get("log_path", "")
                        snapshot["conversation"] = {"channel": conv.get("channel", ""), "conversation_id": conv.get("conversation_id", ""), "display_name": conv.get("display_name", "")}
                        break
        snapshot["log_path"] = log_path
        return snapshot



#!/usr/bin/env python3
from __future__ import annotations

import requests

from _common import API_ROOT, api_request, build_auth_parser, dump, normalize_feishu_event_time, tenant_access_token


def main() -> int:
    parser = build_auth_parser("Update Feishu calendar event")
    parser.add_argument("--calendar-id", required=True)
    parser.add_argument("--event-id", required=True)
    parser.add_argument("--summary", default="")
    parser.add_argument("--start", default="", help="RFC3339 timestamp with timezone offset or unix timestamp seconds")
    parser.add_argument("--end", default="", help="RFC3339 timestamp with timezone offset or unix timestamp seconds")
    parser.add_argument("--timezone", default="Asia/Shanghai")
    parser.add_argument("--description", default="")
    args = parser.parse_args()
    token = tenant_access_token(args.app_id, args.app_secret)
    payload = {}
    for key, value in {
        "summary": args.summary,
        "description": args.description,
    }.items():
        if value:
            payload[key] = value
    if args.start:
        payload["start_time"] = normalize_feishu_event_time(args.start, args.timezone)
    if args.end:
        payload["end_time"] = normalize_feishu_event_time(args.end, args.timezone)
    if not payload:
        raise SystemExit("at least one of --summary/--start/--end/--description is required")
    result = api_request(
        "PATCH",
        f"{API_ROOT}/calendar/v4/calendars/{args.calendar_id}/events/{args.event_id}",
        token=token,
        json=payload,
    )
    dump(result)
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
from __future__ import annotations

import json
import threading
from contextlib import contextmanager
from pathlib import Path
from typing import Any, Callable, Dict

try:
    import fcntl
except ImportError:  # pragma: no cover
    fcntl = None

try:
    from .cheapclaw_runtime_state import CheapClawPaths, ensure_runtime_layout, load_panel_for_paths, save_panel_for_paths, now_iso
    from .tool_runtime_helpers import ensure_conversation, refresh_conversation_context_file
except ImportError:
    from cheapclaw_runtime_state import CheapClawPaths, ensure_runtime_layout, load_panel_for_paths, save_panel_for_paths, now_iso
    from tool_runtime_helpers import ensure_conversation, refresh_conversation_context_file


class CheapClawPanelStore:
    def __init__(self, paths: CheapClawPaths, history_preview_limit: int = 50):
        self.paths = paths
        self.history_preview_limit = max(1, int(history_preview_limit))
        self._thread_lock = threading.RLock()
        self.ensure_layout()

    def ensure_layout(self) -> None:
        ensure_runtime_layout(self.paths)

    @contextmanager
    def _file_lock(self):
        self.paths.panel_dir.mkdir(parents=True, exist_ok=True)
        with self._thread_lock:
            with open(self.paths.panel_lock_path, "a+", encoding="utf-8") as lock_file:
                if fcntl is not None:
                    fcntl.flock(lock_file.fileno(), fcntl.LOCK_EX)
                try:
                    yield
                finally:
                    if fcntl is not None:
                        fcntl.flock(lock_file.fileno(), fcntl.LOCK_UN)

    def load_panel(self) -> Dict[str, Any]:
        return load_panel_for_paths(self.paths)

    def save_panel(self, panel: Dict[str, Any], *, backup: bool = True) -> Dict[str, Any]:
        with self._file_lock():
            return save_panel_for_paths(self.paths, panel, backup=backup)

    def mutate(self, updater: Callable[[Dict[str, Any]], Dict[str, Any] | None]) -> Dict[str, Any]:
        with self._file_lock():
            current = load_panel_for_paths(self.paths)
            updated = updater(current)
            panel = current if updated is None else updated
            return save_panel_for_paths(self.paths, panel, backup=True)

    def record_social_message(self, **kwargs) -> Dict[str, Any]:
        timestamp = kwargs.get("timestamp") or now_iso()
        channel = str(kwargs.get("channel") or "").strip()
        conversation_id = str(kwargs.get("conversation_id") or "").strip()
        message_text = str(kwargs.get("message_text") or "").strip()
        attachments = kwargs.get("attachments") or []
        if not channel or not conversation_id or (not message_text and not attachments):
            raise ValueError("channel, conversation_id and message_text/attachments are required")

        def _update(panel: Dict[str, Any]) -> Dict[str, Any]:
            conv = ensure_conversation(
                panel,
                channel=channel,
                conversation_id=conversation_id,
                conversation_type=str(kwargs.get("conversation_type") or "group"),
                display_name=kwargs.get("display_name") or conversation_id,
                require_mention=bool(kwargs.get("require_mention", True)),
            )
            event = {
                "message_id": str(kwargs.get("message_id") or "").strip(),
                "timestamp": timestamp,
                "sender_id": str(kwargs.get("sender_id") or "").strip(),
                "sender_name": str(kwargs.get("sender_name") or "").strip(),
                "text": message_text,
                "attachments": attachments,
                "is_mention_to_bot": bool(kwargs.get("is_mention_to_bot", False)),
                "direction": "inbound",
            }
            history_path = Path(conv["message_history_path"])
            history_path.parent.mkdir(parents=True, exist_ok=True)
            with open(history_path, "a", encoding="utf-8") as fh:
                fh.write(json.dumps(event, ensure_ascii=False) + "\n")
            conv.setdefault("messages", []).append(event)
            del conv["messages"][:-self.history_preview_limit]
            conv.setdefault("pending_events", []).append({"type": "social_message", "timestamp": timestamp, "message_id": event["message_id"]})
            conv["updated_at"] = timestamp
            conv["latest_user_message_at"] = timestamp
            conv["unread_event_count"] = len(conv["pending_events"])
            return panel

        panel = self.mutate(_update)
        refresh_conversation_context_file(channel, conversation_id, panel)
        return panel

    def set_main_agent_state(self, *, running: bool, run_id: str = "") -> Dict[str, Any]:
        def _update(panel: Dict[str, Any]) -> Dict[str, Any]:
            state = panel["service_state"]
            state["main_agent_running"] = bool(running)
            if run_id:
                state["main_agent_run_id"] = run_id
            if running:
                state["main_agent_last_started_at"] = now_iso()
            else:
                state["main_agent_last_finished_at"] = now_iso()
            return panel

        return self.mutate(_update)

    def mark_watchdog_tick(self) -> Dict[str, Any]:
        def _update(panel: Dict[str, Any]) -> Dict[str, Any]:
            panel["service_state"]["watchdog_last_run_at"] = now_iso()
            return panel

        return self.mutate(_update)



#!/usr/bin/env python3
from pathlib import Path
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import list_global_skills
from tool_server_lite.tools.file_tools import BaseTool

class CheapClawListGlobalSkillsTool(BaseTool):
    name = "cheapclaw_list_global_skills"
    def execute(self, task_id, parameters):
        skills = list_global_skills()
        return {"status": "success", "output": f"skills={len(skills)}", "skills": skills}



#!/usr/bin/env python3
from pathlib import Path
from infiagent import infiagent
from pathlib import Path
import sys

APP_ROOT = Path(__file__).resolve().parents[2]
if str(APP_ROOT) not in sys.path:
    sys.path.insert(0, str(APP_ROOT))

from tool_runtime_helpers import assert_managed_task_id, extend_task_visible_skills, load_panel, update_conversation_task
from tool_server_lite.tools.file_tools import BaseTool

class CheapClawRevealSkillsTool(BaseTool):
    name = "cheapclaw_reveal_skills"
    def execute(self, task_id, parameters):
        try:
            target_task_id = assert_managed_task_id(str(Path(parameters.get("task_id") or "").expanduser().resolve()))
        except ValueError as exc:
            return {"status": "error", "output": "", "error": str(exc)}
        skill_names = parameters.get("skill_names") or []
        if not target_task_id or not isinstance(skill_names, list) or not skill_names:
            return {"status": "error", "output": "", "error": "task_id and non-empty skill_names are required"}
        payload = extend_task_visible_skills(target_task_id, skill_names)
        panel = load_panel()
        for channel_payload in panel.get("channels", {}).values():
            for conv in channel_payload.get("conversations", {}).values():
                for linked in conv.get("linked_tasks", []):
                    if linked.get("task_id") == target_task_id:
                        update_conversation_task(
                            str(conv.get("channel") or ""),
                            str(conv.get("conversation_id") or ""),
                            target_task_id,
                            {"default_exposed_skills": list(payload.get("visible_skills") or [])},
                        )
                        break
        result = {
            "status": "success",
            "output": f"visible skills updated: {len(payload['visible_skills'])}",
            "task_id": target_task_id,
            "visible_skills": list(payload.get("visible_skills") or []),
        }
        if bool(parameters.get("run_fresh", False)):
            agent = infiagent(user_data_root=str(Path(__import__('os').environ.get('MLA_USER_DATA_ROOT','~/mla_v3')).expanduser().resolve()))
            result["fresh"] = agent.fresh(task_id=target_task_id, reason="skills revealed")
        return result



#!/usr/bin/env python3
from __future__ import annotations

from _common import API_ROOT, api_request, build_auth_parser, dump, tenant_access_token


def main() -> int:
    parser = build_auth_parser("Create Feishu doc")
    parser.add_argument("--title", required=True)
    args = parser.parse_args()
    token = tenant_access_token(args.app_id, args.app_secret)
    payload = api_request(
        "POST",
        f"{API_ROOT}/docx/v1/documents",
        token=token,
        json={"title": args.title},
    )
    document = ((payload.get("data") or {}).get("document") or {}) if isinstance(payload, dict) else {}
    document_id = str(document.get("document_id") or "").strip()
    if document_id:
        document["url_hint"] = f"https://wcntovzrui4f.feishu.cn/docx/{document_id}"
        payload.setdefault("data", {})["document"] = document
    dump(payload)
    return 0


if __name__ == "__main__":
    raise SystemExit(main())
